@@ -31,7 +31,8 @@ def isvector(m):
 
 
 def hermitian(m):
-    return np.transpose(np.conjugate(m))
+    return m.conjugate().transpose() 
+#    return np.transpose(np.conjugate(m))
 
 H = hermitian # alias
 get_dagger = hermitian
@@ -77,7 +78,7 @@ def todense(m):
     if issparse(m):
         if m.shape[0]<maxsize: return np.array(m.todense())
         else: raise
-    else: return np.array(m,dtype=np.complex)
+    else: return np.array(m,dtype=np.complex128)
 
 
 def braket_ww(w,wi):
@@ -118,7 +119,7 @@ def get_representation(wfs,A):
     Gets the matrix representation of a certain operator
     """
     n = len(wfs) # number of eigenfunctions
-    ma = np.zeros((n,n),dtype=np.complex) # representation of A
+    ma = np.zeros((n,n),dtype=np.complex128) # representation of A
     A = np.array(A)
     for i in range(n):
       vi = wfs[i] # first wavefunction
@@ -140,7 +141,7 @@ def todouble(vs,ind):
     """Double the eigenvectors"""
     nv = vs.shape[0]
     dim = vs.shape[1]
-    vout = np.zeros((dim*2,nv),dtype=np.complex) # output vector
+    vout = np.zeros((dim*2,nv),dtype=np.complex128) # output vector
     return todouble_jit(vs,ind,vout,nv,dim)
 
 @jit(nopython=True)
@@ -150,7 +151,8 @@ def todouble_jit(vs,ind,vout,nv,dim):
         vout[2*i+ind,:] = vs[i,:]
     return vout
 
-
+eig = dlg.eig # non Hermitian diagonalization
+eigvals = dlg.eigvals # non Hermitian diagonalization
 
 accelerate = False 
 
@@ -220,13 +222,13 @@ def matrix2vector(v):
 
 
 
-def smalleig(m,numw=10,evecs=False,tol=arpack_tol):
+def smalleig(m,numw=10,evecs=False,e0=0.,tol=arpack_tol):
     """
     Return the smallest eigenvalues using arpack
     """
     m = csc_matrix(m) # sparse matrix
     try:
-        eig,eigvec = slg.eigsh(m,k=numw,which="LM",sigma=0.0,
+        eig,eigvec = slg.eigsh(m,k=numw,which="LM",sigma=e0,
                                         tol=tol)
         if evecs:  return eig,eigvec.transpose()  # return eigenvectors
         else:  return eig  # return eigenvalues
@@ -273,8 +275,8 @@ def sqrtm_rotated(M,positive=True):
     (evals,evecs) = dlg.eigh(M) # eigenvals and eigenvecs
     if positive:
         if np.min(evals)<0.:
-            print("Matrix is not positive defined")
-            evals[evals<0.] = 1e-7
+#            print("Matrix is not positive defined",print(evals[evals<0.]))
+            evals[evals<0.] = 0.
     evecs = dagger(np.matrix(evecs)) # change of basis
     m2 = np.matrix([[0.0j for i in evals] for j in evals]) # create matrix
     for i in range(len(evals)):
@@ -316,3 +318,18 @@ def applyinverse(A,b):
 
 
 
+def is_zero(m):
+    """Check if a matrix is zero"""
+    m = todense(m)
+    return np.max(np.abs(m))<1e-6
+
+
+def bmat(M):
+    from scipy.sparse import bmat
+    if len(M)==1: 
+        return todense(M[0][0])
+    else: return todense(bmat(M))
+
+def expm(M):
+    M = todense(M)
+    return dlg.expm(M)
