@@ -68,93 +68,103 @@ def ket_Aw(A,w):
 
 
 
+def get_bands(self,**kwargs):
+    """Compute n-dimensional bandstructures"""
+    if self.non_hermitian: # non Hermitian Hamiltonians
+        from .nonhermitiantk.nhmethods import get_bands as get_bands_NH
+        return get_bands_NH(self,**kwargs)
+    else: # for hermitian Hamiltonians
+        return get_bands_nd(self,**kwargs)
+
+
+
 
 def get_bands_nd(h,kpath=None,operator=None,num_bands=None,
                     callback=None,central_energy=0.0,nk=400,
                     ewindow=None,
                     output_file="BANDS.OUT",write=True,
                     silent=True):
-  """
-  Get an n-dimensional bandstructure
-  """
-  if num_bands is not None:
-    if num_bands>(h.intra.shape[0]-1): num_bands=None
-  if operator is not None: operator = h.get_operator(operator)
-  if num_bands is None: # all the bands
-    if operator is not None: 
-      def diagf(m): # diagonalization routine
-          return algebra.eigh(m) # all eigenvals and eigenfuncs
-    else: 
-      def diagf(m): # diagonalization routine
-          return algebra.eigvalsh(m) # all eigenvals and eigenfuncs
-  else: # using arpack
-    h = h.copy()
-    h.turn_sparse() # sparse Hamiltonian
-    def diagf(m):
-      eig,eigvec = slg.eigsh(m,k=num_bands,which="LM",sigma=central_energy,
-                                  tol=arpack_tol,maxiter=arpack_maxiter)
-      if operator is None: return eig
-      else: return (eig,eigvec)
-  # open file and get generator
-  hkgen = h.get_hk_gen() # generator hamiltonian
-  kpath = h.geometry.get_kpath(kpath,nk=nk) # generate kpath
-  def getek(k):
-    """Compute this k-point"""
-    out = "" # output string
-    hk = hkgen(kpath[k]) # get hamiltonian
-    if operator is None:
-      es = diagf(hk)
-      es = np.sort(es) # sort energies
-      for e in es:  # loop over energies
-        out += str(k)+"   "+str(e)+"\n" # write in file
-      if callback is not None: callback(k,es) # call the function
-    else:
-      es,ws = diagf(hk)
-      ws = ws.transpose() # transpose eigenvectors
-      def evaluate(w,k,A): # evaluate the operator
-          if type(A)==operators.Operator:
-              waw = A.braket(w,k=kpath[k]).real
-          elif callable(A):  
-            try: waw = A(w,k=kpath[k]) # call the operator
-            except: 
-              print("Check out the k optional argument in operator")
-              waw = A(w) # call the operator
-          else: waw = braket_wAw(w,A).real # calculate expectation value
-          return waw # return the result
-      for (e,w) in zip(es,ws):  # loop over waves
-        if callable(ewindow):
-            if not ewindow(e): continue # skip iteration
-        if isinstance(operator, (list,)): # input is a list
-            waws = [evaluate(w,k,A) for A in operator]
-        else: waws = [evaluate(w,k,operator)]
-        out += str(k)+"   "+str(e)+"  " # write in file
-        for waw in waws:  out += str(waw)+"  " # write in file
-        out += "\n" # next line
-      # callback function in each iteration
-      if callback is not None: callback(k,es,ws) # call the function
-    return out # return string
-  ### Now evaluate the function
-  from . import parallel
-  if write: f = open(output_file,"w") # open bands file
-  if parallel.cores==1: ### single thread ###
-    tr = timing.Testimator("BANDSTRUCTURE",silent=silent) # generate object
-    esk = "" # empty list
-    for k in range(len(kpath)): # loop over kpoints
-      tr.remaining(k,len(kpath)) # estimate of the time
-      ek = getek(k)
-      esk += ek # store
-      if write: f.write(ek) # write this kpoint
-      if write: f.flush() # flush in file
-  else: # parallel run
-      esk = parallel.pcall(getek,range(len(kpath))) # compute all
-      esk = "".join(esk) # concatenate all
-      if write: f.write(esk)
-  if write: f.close()
-#  print("\nBANDS finished")
-  esk = esk.split("\n") # split
-  del esk[-1] # remove last one
-  esk = np.array([[float(i) for i in ek.split()] for ek in esk]).T
-  return esk # return data
+    """
+    Get an n-dimensional bandstructure
+    """
+    if num_bands is not None:
+      if num_bands>(h.intra.shape[0]-1): num_bands=None
+    if operator is not None: operator = h.get_operator(operator)
+    if num_bands is None: # all the bands
+      if operator is not None: 
+        def diagf(m): # diagonalization routine
+            return algebra.eigh(m) # all eigenvals and eigenfuncs
+      else: 
+        def diagf(m): # diagonalization routine
+            return algebra.eigvalsh(m) # all eigenvals and eigenfuncs
+    else: # using arpack
+      h = h.copy()
+      h.turn_sparse() # sparse Hamiltonian
+      def diagf(m):
+        eig,eigvec = slg.eigsh(m,k=num_bands,which="LM",sigma=central_energy,
+                                    tol=arpack_tol,maxiter=arpack_maxiter)
+        if operator is None: return eig
+        else: return (eig,eigvec)
+    # open file and get generator
+    hkgen = h.get_hk_gen() # generator hamiltonian
+    kpath = h.geometry.get_kpath(kpath,nk=nk) # generate kpath
+    def getek(k):
+      """Compute this k-point"""
+      out = "" # output string
+      hk = hkgen(kpath[k]) # get hamiltonian
+      if operator is None:
+        es = diagf(hk)
+        es = np.sort(es) # sort energies
+        for e in es:  # loop over energies
+          out += str(k)+"   "+str(e)+"\n" # write in file
+        if callback is not None: callback(k,es) # call the function
+      else:
+        es,ws = diagf(hk)
+        ws = ws.transpose() # transpose eigenvectors
+        def evaluate(w,k,A): # evaluate the operator
+            if type(A)==operators.Operator:
+                waw = A.braket(w,k=kpath[k]).real
+            elif callable(A):  
+              try: waw = A(w,k=kpath[k]) # call the operator
+              except: 
+                print("Check out the k optional argument in operator")
+                waw = A(w) # call the operator
+            else: waw = braket_wAw(w,A).real # calculate expectation value
+            return waw # return the result
+        for (e,w) in zip(es,ws):  # loop over waves
+          if callable(ewindow):
+              if not ewindow(e): continue # skip iteration
+          if isinstance(operator, (list,)): # input is a list
+              waws = [evaluate(w,k,A) for A in operator]
+          else: waws = [evaluate(w,k,operator)]
+          out += str(k)+"   "+str(e)+"  " # write in file
+          for waw in waws:  out += str(waw)+"  " # write in file
+          out += "\n" # next line
+        # callback function in each iteration
+        if callback is not None: callback(k,es,ws) # call the function
+      return out # return string
+    ### Now evaluate the function
+    from . import parallel
+    if write: f = open(output_file,"w") # open bands file
+    if parallel.cores==1: ### single thread ###
+      tr = timing.Testimator("BANDSTRUCTURE",silent=silent) # generate object
+      esk = "" # empty list
+      for k in range(len(kpath)): # loop over kpoints
+        tr.remaining(k,len(kpath)) # estimate of the time
+        ek = getek(k)
+        esk += ek # store
+        if write: f.write(ek) # write this kpoint
+        if write: f.flush() # flush in file
+    else: # parallel run
+        esk = parallel.pcall(getek,range(len(kpath))) # compute all
+        esk = "".join(esk) # concatenate all
+        if write: f.write(esk)
+    if write: f.close()
+  #  print("\nBANDS finished")
+    esk = esk.split("\n") # split
+    del esk[-1] # remove last one
+    esk = np.array([[float(i) for i in ek.split()] for ek in esk]).T
+    return esk # return data
 
 
 
