@@ -3,16 +3,14 @@ import numpy as np
 from numba import jit
 from . import parallel
 
-#try:
-#  from . import density_matrixf90
-#  use_fortran = True
-#except:
-#  use_fortran = False
-##  print("Fortran routines not working in densitymatrix.py")
- 
+delta_dm = 1e-6 # default energy smearing
+
+
 use_fortran = False
 
-def full_dm(h,use_fortran=use_fortran,nk=10,fermi=0.0,delta=1e-6,ds=None):
+def full_dm(h,use_fortran=use_fortran,nk=10,fermi=0.0,
+        delta=delta_dm,
+        ds=None):
   if h.dimensionality == 0: fac = 1.
   elif h.dimensionality == 1: fac = 1./nk
   elif h.dimensionality == 2: fac = 1./nk**2
@@ -21,11 +19,11 @@ def full_dm(h,use_fortran=use_fortran,nk=10,fermi=0.0,delta=1e-6,ds=None):
   if ds is None: # no directions required
     es,vs = h.get_eigenvectors(nk=nk) # get eigenvectors
     es = es - fermi # shift by the Fermi energy
-    if use_fortran:
-      dm = density_matrixf90.density_matrix(np.array(es),np.array(vs),delta)
-      return dm*fac
-    else:
-      return np.matrix(full_dm_python(h.intra.shape[0],es,np.array(vs),
+#    if use_fortran:
+#      dm = density_matrixf90.density_matrix(np.array(es),np.array(vs),delta)
+#      return dm*fac
+#    else:
+    return np.matrix(full_dm_python(h.intra.shape[0],es,np.array(vs),
                            delta=delta))*fac # call the function
   else: # directions required
     es,vs,ks = h.get_eigenvectors(nk=nk,kpoints=True) # get eigenvectors
@@ -43,13 +41,13 @@ def full_dm(h,use_fortran=use_fortran,nk=10,fermi=0.0,delta=1e-6,ds=None):
 
 def full_dm_python(n,es,vs,delta=1e-6):
   """Calculate the density matrix"""
-  dm = np.zeros((n,n),dtype=np.complex)
+  dm = np.zeros((n,n),dtype=np.complex128)
   return full_dm_python_jit(n,es,vs,dm,delta)
 
 
 def full_dm_python_d(n,es,vs,ks,d):
   """Calculate the density matrix"""
-  dm = np.zeros((n,n),dtype=np.complex)
+  dm = np.zeros((n,n),dtype=np.complex128)
   return full_dm_python_d_jit(n,es,vs,ks,np.array(d),dm)
 
 
@@ -91,7 +89,7 @@ def full_dm_python_d_jit(n,es,vs,ks,d,dm):
 
 
 
-def restricted_dm(h,use_fortran=True,mode="KPM",pairs=[],
+def restricted_dm(h,use_fortran=False,mode="KPM",pairs=[],
                    scale=10.0,npol=400,ne=None):
   """Calculate certain elements of the density matrix"""
   if h.dimensionality != 0 : raise
@@ -103,7 +101,7 @@ def restricted_dm(h,use_fortran=True,mode="KPM",pairs=[],
     if ne is None: ne = npol*4
     from . import kpm
     xin = np.linspace(-.99*scale,0.0,ne) # input x array
-    out = np.zeros(len(pairs),dtype=np.complex)
+    out = np.zeros(len(pairs),dtype=np.complex128)
     ii = 0
     for (i,j) in pairs: # loop over inputs
       (x,y) = kpm.dm_ij_energy(h.intra,i=i,j=j,scale=scale,npol=npol,
