@@ -15,13 +15,16 @@ from .hamiltonians import Hamiltonian
 
 class Embedding():
     """Define an embedding object"""
-    def __init__(self,h,m=None,nsuper=None):
+    def __init__(self,h,m=None,nsuper=None,selfenergy=None):
         self.H = h.copy() # Pristine Hamiltonian
         self.geometry = self.H.geometry
         self.dimensionality = self.H.dimensionality
+        self.selfenergy = selfenergy 
         self.has_gf_generator = False
         self.has_spin = self.H.has_spin
         self.has_eh = self.H.has_eh
+        self.gf_mode = "renormalization" # mode for Green's functions
+        self.boundary_embedding_generator = None
         self.mode = "bulk" # mode
         self.nsuper = None # supercell between original Hamiltonian
         if m is not None: 
@@ -48,32 +51,22 @@ class Embedding():
         return get_dm(self,**kwargs)
     def get_ldos(self,**kwargs):
         return get_ldos(self,**kwargs)
-    def ldos(self,**kwargs): return self.get_ldos(**kwargs)
-    def dos(self,**kwargs):
-        (x,y,d) = self.ldos(**kwargs)
+    def ldos(self,**kwargs): 
+        return self.get_ldos(**kwargs)
+    def get_dos(self,**kwargs):
+        (x,y,d) = self.get_ldos(**kwargs)
         return np.sum(d) # sum the DOS
+    def dos(self,**kwargs): 
+        return self.get_dos(**kwargs)
     def multidos(self,es=np.linspace(-1.0,1.0,30),
                    energies=None,**kwargs):
         if energies is not None: es = energies # overwrite
         ds = parallel.pcall(lambda e: self.dos(energy=e,**kwargs),es)
         return (es,np.array(ds))
-    def multildos(self,es=np.linspace(-2.,2.,20),**kwargs):
+    def multildos(self,**kwargs):
         """Compute the ldos at different energies"""
-        fs.rmdir("MULTILDOS")
-        fs.mkdir("MULTILDOS")
-        ds = [] # total DOS
-        fo = open("MULTILDOS/MULTILDOS.TXT","w")
-        # parallel execution
-        out = parallel.pcall(lambda x: self.ldos(energy=x,**kwargs),es) 
-        for (e,o) in zip(es,out):
-            (x,y,d) = o # extract
-#            (x,y,d) = self.ldos(e=e,**kwargs) # compute LDOS
-            ds.append(np.mean(d)) # total DOS
-            name0 = "LDOS_"+str(e)+"_.OUT" # name
-            name = "MULTILDOS/"+name0
-            fo.write(name0+"\n") # name of the file
-            np.savetxt(name,np.array([x,y,d]).T) # save data
-        np.savetxt("MULTILDOS/DOS.OUT",np.array([es,ds]).T)
+        from .embeddingtk.ldos import multildos
+        return multildos(self,**kwargs)
     def set_multihopping(self,mh):
         """Set a multihopping as the impurity"""
         dd = mh.get_dict() # get the dictionary
@@ -90,7 +83,8 @@ class Embedding():
     # dummy methods for compatibility
     def turn_multicell(self): pass
     def get_multicell(self): return self
-    def turn_dense(self): pass
+#    def turn_dense(self): pass
+    def get_dense(self): return self
     def get_dict(self): return self.H.get_dict()
     def shift_fermi(self,mu): self.H.shift_fermi(mu)
     def get_total_energy(self,**kwargs): return 0.0
@@ -98,10 +92,14 @@ class Embedding():
         from .selfconsistency.embedding import hubbard_mf
         return hubbard_mf(self,**kwargs) # return Hubbard mean-field
     def get_didv(self,**kwargs):
+        from .embeddingtk import didv
+        return didv.get_didv_single(self,**kwargs)
+    def get_didv_all(self,**kwargs):
         from .embeddingtk import didv
         return didv.get_didv(self,**kwargs)
     # for compatibility with kappa functionality
-    def didv(self,**kwargs): self.get_didv(**kwargs) 
+    def didv(self,**kwargs): 
+        return self.get_didv(**kwargs) 
     def get_kappa(self,**kwargs):
         from .embeddingtk import kappa
         return kappa.get_kappa(self,**kwargs)
@@ -116,7 +114,7 @@ def dos_impurity(h,vc=None,energies=np.linspace(-.5,.5,20),
                    use_generator=False):
   """ Calculates the green function using the embedding technique"""
   if vc is None: vc = h.intra  # assume perfect
-  iden = np.identity(h.intra.shape[0],dtype=np.complex)
+  iden = np.identity(h.intra.shape[0],dtype=np.complex128)
   if use_generator:
     getgreen = green.green_generator(h,nk=nk) # get the function
     def get_green(energy): return getgreen(energy,delta=delta)
@@ -171,8 +169,60 @@ def onsite_defective_central(h,m,nsuper):
     return onsite_supercell(h,nsuper,mc=m)
 
 
-def onsite_supercell(h,nsuper,mc=None):
-    if h.is_multicell: h = h.get_no_multicell() # redefine
+
+def onsite_supercell_multicell(h,nsuper,mc=None):
+    """Compute the onsite matrix of a supercell, with a defect mc"""
+    if nsuper==1: hs = h.copy() # just make a copy
+    else: hs = h.get_supercell(nsuper) # get a supercell of the Hamiltonian
+    # new lets replace the onsite matrix of the defective cell
+    ni = h.intra.shape[0] # dimension of minimal cell
+    nis = hs.intra.shape[0] # dimension of supercell
+    if mc is None: return hs.intra
+    ### setup the central (defective) cell
+    if h.dimensionality==1: n = nsuper # number of unit cells
+    elif h.dimensionality==2: n = nsuper**2 # number of unit cells
+    else: raise
+    if h.dimensionality==1:
+        ic=int(n//2) # central site
+    elif h.dimensionality==2:
+        if nsuper%2==1: # odd supercell
+            ic=int(n//2)
+        else: # even supercell
+            ic=int(n//2) # central
+            ic = ic - int(nsuper//2)
+    ii = ic*ni # first index
+    jj = (ic+1)*ni # last index
+    hs.intra[ii:jj,ii:jj] = mc[:,:] # replace
+    return hs.intra # return intracell
+
+# redefine
+def onsite_supercell(h0,nsuper,**kwargs):
+    """Generic function for supercell"""
+    try: # try to use the non multicell method
+        h = h0.get_no_multicell()
+        multicell_mode = False
+    except: # if not possible, try the multicell one (not well tested yet)
+        h = h0.copy()
+        multicell_mode = True
+    if multicell_mode:
+        print("WARNING, Multicell function in onsite_supercell")
+        return onsite_supercell_multicell(h,nsuper,**kwargs)
+    else:
+        return onsite_supercell_no_multicell(h,nsuper,**kwargs)
+
+
+
+def onsite_supercell_no_multicell(h,nsuper,mc=None):
+    """Compute the onsite matrix of a supercell, with a defect mc"""
+    if nsuper==1: 
+        if mc is None: return h.intra # pristine
+        else: return mc # defective
+    if h.is_multicell: # try to make it multicell 
+        from .htk.kchain import detect_longest_hopping
+        if detect_longest_hopping(h)>1:
+            print("This function requires short-range hopping, stopping")
+            raise # up to NN
+        h = h.get_no_multicell() # redefine
     from .checkclass import is_iterable
     if not is_iterable(nsuper): # just a number 
         if h.dimensionality==1: nsuper = [nsuper,1]
@@ -204,53 +254,66 @@ def onsite_supercell(h,nsuper,mc=None):
     intra = csc(h.intra)
     if mc is None: mc = intra
     else: mc = csc(mc)
+    dag = algebra.dagger
     for i in range(n):
-      intrasuper[i][i] = intra # intracell
-      (x1,y1) = inds[i]
-      for j in range(n):
-        (x2,y2) = inds[j]
-        dx = x2-x1
-        dy = y2-y1
-        if dx==1 and  dy==0: intrasuper[i][j] = tx
-        if dx==-1 and dy==0: intrasuper[i][j] = tx.H
-        if dx==0 and  dy==1: intrasuper[i][j] = ty
-        if dx==0 and  dy==-1: intrasuper[i][j] = ty.H
-        if dx==1 and  dy==1: intrasuper[i][j] = txy
-        if dx==-1 and dy==-1: intrasuper[i][j] = txy.H
-        if dx==1 and  dy==-1: intrasuper[i][j] = txmy
-        if dx==-1 and dy==1: intrasuper[i][j] = txmy.H
-    if nsuper[0]%2==1: # central cell
-        ii=int(n/2)
-        intrasuper[ii][ii] = mc # central onsite
-    else:
-        ii=int(n/2)
-        ii = ii - int(nsuper[0]/2)
+        intrasuper[i][i] = intra # intracell
+        (x1,y1) = inds[i]
+        for j in range(n):
+            (x2,y2) = inds[j]
+            dx = x2-x1
+            dy = y2-y1
+            if dx==1 and  dy==0: intrasuper[i][j] = tx
+            if dx==-1 and dy==0: intrasuper[i][j] = dag(tx)
+            if dx==0 and  dy==1: intrasuper[i][j] = ty
+            if dx==0 and  dy==-1: intrasuper[i][j] = dag(ty)
+            if dx==1 and  dy==1: intrasuper[i][j] = txy
+            if dx==-1 and dy==-1: intrasuper[i][j] = dag(txy)
+            if dx==1 and  dy==-1: intrasuper[i][j] = txmy
+            if dx==-1 and dy==1: intrasuper[i][j] = dag(txmy)
+    ### setup the central (defective) cell
+    if h.dimensionality==1:
+        ii=int(n//2) # central site
         intrasuper[ii][ii] = mc # central onsite
+    elif h.dimensionality==2:
+        if nsuper[0]%2==1: # odd supercell
+            ii=int(n//2)
+            intrasuper[ii][ii] = mc # central onsite
+        else: # even supercell
+            ii=int(n//2) # central
+            ii = ii - int(nsuper[0]//2)
+            intrasuper[ii][ii] = mc # central onsite
+    else: raise
     intrasuper = bmat(intrasuper).todense() # supercell
     return intrasuper
 
 
+def get_gf(self,**kwargs):
+    """Return the Green's function"""
+    if self.selfenergy is None: # no selfenergy given, compute it
+        return get_gf_exact(self,**kwargs)
+    else: # a selfenergy is given
+        from .embeddingtk.boundaryembedding import boundary_embedding_gf
+        return boundary_embedding_gf(self,selfenergy=self.selfenergy,**kwargs)
 
 
-
-
-def get_gf(self,energy=0.0,delta=1e-2,nsuper=1,nk=100,operator=None,**kwargs):
+def get_gf_exact(self,energy=0.0,delta=1e-2,
+        nsuper=1,nk=100,operator=None,**kwargs):
     """Return the Green's function"""
     h = self.H
     e = energy
     if self.nsuper is None: # old way
         g,selfe = green.supercell_selfenergy(h,e=e,delta=delta,nk=nk,
-                nsuper=nsuper,gtype=self.mode)
+                nsuper=nsuper,gtype=self.mode,gf_mode=self.gf_mode)
     else: # workaround for supercells
         raise # this does not work yet
         g,selfe = green.supercell_selfenergy(h,e=e,delta=delta,nk=nk,
                 nsuper=nsuper*self.nsuper) # compute Green's function
         h = h.supercell(self.nsuper) # and redefine with a supercell
     ms = onsite_supercell(h,nsuper)
-    n = self.m.shape[0]
+    n = self.m.shape[0] # dimension of the matrix
     ms = onsite_defective_central(h,self.m,nsuper)
     ns = ms.shape[0] # dimension of the supercell
-    iden = np.identity(ns,dtype=np.complex) # identity
+    iden = np.identity(ns,dtype=np.complex128) # identity
     emat = iden*(e + delta*1j) # energy matrix
     gv = algebra.inv(emat - ms -selfe)   # Defective Green function
     return gv
@@ -266,26 +329,10 @@ def get_onsite(self,nsuper=1,**kwargs):
 from .embeddingtk.embedded import get_dm
 from .embeddingtk.embedded import embed_hamiltonian
 from .embeddingtk.selfenergies import get_selfenergy_from_potential
+from .embeddingtk.ldos import get_ldos
 
 
 
-def get_ldos(self,energy=0.0,delta=1e-2,nsuper=1,nk=100,
-                    write = True,
-                    operator=None,**kwargs):
-    """Compute the local density of states"""
-    h = self.H
-    # get the Green's function
-    gv = self.get_gf(energy=energy,delta=delta,nsuper=nsuper,nk=nk)
-    if operator is not None:
-        operator = h.get_operator(operator) # overwrite
-        gv = operator*gv # multiply
-    ds = [-gv[i,i].imag/np.pi for i in range(gv.shape[0])] # LDOS
-    ds = full2profile(h,ds,check=False) # resum if necessary
-    ds = np.array(ds) # convert to array
-    gs = h.geometry.supercell(nsuper)
-    x,y = gs.x,gs.y
-    if write: np.savetxt("LDOS.OUT",np.array([x,y,ds]).T)
-    return x,y,ds
 
 
 
