@@ -2,7 +2,7 @@ import numpy as np
 from scipy.interpolate import interp1d
 from scipy.optimize import minimize
 import warnings
-warnings.filterwarnings("ignore", category=np.VisibleDeprecationWarning) 
+#warnings.filterwarnings("ignore", category=np.VisibleDeprecationWarning) 
 # suppress the numpy warning on arrays
 
 
