@@ -11,7 +11,7 @@ def spin_channel(m,spin_column=None,spin_row=None,has_spin=True):
   if (spin_row is None) or (spin_column is None): return m # return initial
   n = m.shape[0] # shape of the matrix
   n2 = n//2 # number of orbitals
-  out = np.zeros((n,n),dtype=np.complex)
+  out = np.zeros((n,n),dtype=np.complex128)
   if spin_column=="up": ii = 0
   else: ii = 1
   if spin_row=="up": jj = 0
@@ -26,7 +26,7 @@ def swave(m):
     """Extract the swave pairing from a matrix, assuming
     the Nambu spinor basis"""
     n = m.shape[0]//4 # number of sites
-    ds = np.zeros(n,dtype=np.complex) # pairing
+    ds = np.zeros(n,dtype=np.complex128) # pairing
     for i in range(n):
       ds[i] = m[4*i,4*i+2] # get the pairing
     return ds
@@ -138,13 +138,17 @@ def hopping_spinless(m,cutoff=0.001):
 def extract_from_hamiltonian(self,name,**kwargs):
     """Extract a quantity from a Hamiltonian"""
     h0 = self.copy()
-    if self.is_sparse: h0.turn_dense() # turn into dense form
-    if name=="density":
+    if self.is_sparse: h0 = h0.get_dense() # turn into dense form
+    if name in ["density","onsite"]:
       if self.has_eh: 
           h0.remove_nambu()
           m = h0.intra
       else: m = self.intra
-      return onsite(m,has_spin=self.has_spin)
+      if not self.non_hermitian: # Hermitian case
+          return onsite(m,has_spin=self.has_spin)
+      else: # non Hermitian case
+          from .nonhermitiantk.extract import onsite as onsite_NH
+          return onsite_NH(m,has_spin=self.has_spin)
     elif name=="mx" and self.has_spin:
       if self.has_eh: h0.remove_nambu() # not implemented
       return mx(h0.intra)
@@ -178,6 +182,12 @@ def extract_from_hamiltonian(self,name,**kwargs):
     elif name=="deltak": # reciprocal space superconductivity
         from .sctk.extract import extract_pairing_kmap
         return extract_pairing_kmap(self,**kwargs)
+    elif name=="absolute_delta": # reciprocal space superconductivity
+        from .sctk.extract import extract_absolute_pairing
+        return extract_absolute_pairing(self,**kwargs)
+    elif name=="absolute_spatial_delta": 
+        from .sctk.extract import extract_absolute_spatial_pairing
+        return extract_absolute_spatial_pairing(self,**kwargs)
     else: raise
 
 
@@ -191,7 +201,7 @@ def extract_onsite_matrix_function(h,**kwargs):
       if 2*n!=m.shape[0]: raise
       def f(r):
         ind = h.geometry.get_index(r,**kwargs) # get the index
-        if ind is None: return np.zeros((2,2),dtype=np.complex)
+        if ind is None: return np.zeros((2,2),dtype=np.complex128)
         else: return m[2*ind:2*ind+2,2*ind:2*ind+2]
     return f # return function
 
