@@ -13,13 +13,6 @@ from numba import jit
 from .htk.g2h import get_hamiltonian
 from .helptk import get_docstring
 
-
-try:
-  from . import supercellf90
-  use_fortran = True
-except: 
-  use_fortran = False
-#  print("FORTRAN routines not present in geometry.py")
 use_fortran = False
 
 class Geometry:
@@ -128,9 +121,9 @@ class Geometry:
         self.xyz2r() # update r
     def center(self):
         """ Centers the geometry in (0,0,0)"""
-        self.x = self.x - sum(self.x)/len(self.x)
-        self.y = self.y - sum(self.y)/len(self.y)
-        self.z = self.z - sum(self.z)/len(self.z)
+        self.x = self.x - np.sum(self.x)/len(self.x)
+        self.y = self.y - np.sum(self.y)/len(self.y)
+        self.z = self.z - np.sum(self.z)/len(self.z)
         self.xyz2r() # update r
     def get_lattice_name(self):
         if self.dimensionality==2:
@@ -197,6 +190,7 @@ class Geometry:
         from .geometrytk.replicas import multireplicas
         return multireplicas(self,n)
     def bloch_phase(self,d,k):
+        """Return the Bloch's phase for a specific k-vector"""
         from .geometrytk.bloch import bloch_phase
         return bloch_phase(self,d,k)
     def remove(self,i=0):
@@ -308,7 +302,7 @@ def ladder(): return square_ribbon(2)
 
 def bisquare_ribbon(ncells):
   g = square_lattice_bipartite()
-  g = g.supercell((1,ncells))
+  g = g.get_supercell((1,ncells))
   g.dimensionality = 1
   return g
 
@@ -580,9 +574,23 @@ def triangular_lattice_tripartite():
   """
   Creates a triangular lattice with three sites per unit cell
   """
-  g = triangular_lattice()
-  return supercelltk.target_angle_volume(g,angle=1./3.,volume=3,
-          same_length=True)
+  rs = [] # empty list
+  rs.append([0.,0.,0.]) # first position
+  rs.append([1.,0.,0.]) # second position
+  rs.append([1./2.,np.sqrt(3.)/2.,0.]) # third position
+  rs = np.array(rs)
+  g = Geometry() # create geometry
+  g.r = np.array(rs) # store array
+  g.r2xyz() # tranform
+  g.has_sublattice = False # has sublattice index
+  g.a1 = np.array(rs[1]+rs[2])
+  g.a2 = np.array(-rs[1]+2*rs[2])
+  g.dimensionality = 2 # two dimensional system
+  g.get_fractional() # update reciprocal lattice vectors
+  return g
+#  g = triangular_lattice()
+#  return supercelltk.target_angle_volume(g,angle=1./3.,volume=3,
+#          same_length=True)
 
 
 
@@ -760,6 +768,7 @@ def kagome_lattice(n=1):
   if n>1: return supercelltk.target_angle(g,angle=1./3.,volume=int(n),
           same_length=True) 
   g.update_reciprocal()
+  g.get_fractional()
   return g
 
 
@@ -836,7 +845,9 @@ def read(input_file="POSITIONS.OUT"):
     if len(g.sublattice) != len(g.r): raise
     print("Read sublattice from SUBLATTICE.OUT")
   except: g.has_sublattice = False
-  dim = int(open("DIMENSIONALITY.OUT").read())
+  try:
+    dim = int(open("DIMENSIONALITY.OUT").read())
+  except: dim = 0
   g.dimensionality = dim # store
   if dim>0: # if it has lattice
     lat = np.genfromtxt("LATTICE.OUT")   # read lattice
@@ -1120,7 +1131,7 @@ def write_profile(g,d,name="PROFILE.OUT",nrep=3,normal_order=False):
   """Write a certain profile in a file"""
   if g.dimensionality == 0: nrep = 1
   if callable(d): d = np.array([d(ri) for ri in g.r]) # call
-  else: d = np.array(d)
+  else: d = np.array(d) # assume it is an array
   go = g.copy() # copy geometry
   go = go.supercell(nrep) # create supercell
   d = replicate_array(g,d,nrep=nrep) # replicate
@@ -1130,7 +1141,7 @@ def write_profile(g,d,name="PROFILE.OUT",nrep=3,normal_order=False):
   else:
       m = np.array([go.x,go.y,d,go.z]).T
       header = "x        y     profile      z"
-  np.savetxt(name,m,fmt='%.5f',delimiter="    ",header=header) # save in file
+  np.savetxt(name,m,fmt='%.8f',delimiter="    ",header=header) # save in file
 
 
 
