@@ -9,8 +9,6 @@ from .algebra import dagger
 
 dag = dagger
 
-
-
 class Heterostructure():
     """Class for a HTstructure"""
     def copy(self):
@@ -69,11 +67,11 @@ class Heterostructure():
         return device_dos(self,mode="left",**kwargs)
     def get_coupled_right_dos(self,**kwargs):
         return device_dos(self,mode="right",**kwargs)
-    def landauer(self,energy=[0.],delta=0.0001,do_leads=True,left_channel=None,
-                  right_channel=None):
+    def landauer(self,energy=0.,do_leads=True,left_channel=None,
+                  right_channel=None,**kwargs):
       """ Return the Landauer transmission"""
       if self.has_eh: raise # invalid if there is electorn-hole
-      return landauer(self,energy=energy,delta=delta,do_leads=do_leads,
+      return landauer(self,energy=energy,delta=self.delta,do_leads=do_leads,
                       left_channel=left_channel,right_channel=right_channel)
     def write_green(self):
         """Writes the green functions in a file"""
@@ -244,7 +242,7 @@ def create_leads_and_central(h_right,h_left,h_central,num_central=1,
     for i in range(num_central-1): # interterm of the central blocks
       r = float(i+1)/float(num_central+1)   # 
       hc[i][i+1] = tr*r + tl*(1.-r)
-      hc[i+1][i] = (tr*r + tl*(1.-r)).H
+      hc[i+1][i] = dagger(tr*r + tl*(1.-r))
 
 
   else:
@@ -274,6 +272,8 @@ def create_leads_and_central(h_right,h_left,h_central,num_central=1,
   ht.central_intra = hc
   # and modify the geometry
   ht.central_geometry.supercell(num_central) 
+  ht.Hr = h_right.copy()
+  ht.Hl = h_left.copy()
   return ht
 
 HTstructure = Heterostructure
