@@ -32,7 +32,7 @@ def get_index(g,r,replicas=False):
     """Given a certain position, return the index of it in the geometry"""
     if replicas: # check the replicas
       ds = g.neighbor_directions()
-      rset = [g.replicas(d) for d in ds]  # all sets of replicas
+      rset = [g.replicas(d=d) for d in ds]  # all sets of replicas
     else:
       rset = [g.r] # list of positions
     for rs in rset: # loop over set of sites
@@ -42,7 +42,7 @@ def get_index(g,r,replicas=False):
     return None # not found
 
 
-@jit
+@jit(nopython=True)
 def get_index_jit(r0,rs):
     for i in range(len(rs)): # loop
         dr = r0 - rs[i] # different
