@@ -5,66 +5,54 @@ import numpy as np
 from numba import jit
 from . import parallel
 
-try:
-  from . import kanemelef90
-  use_fortran = True
-except:
-#  print("Not possible to use FORTRAN rutines, kanemele.py")
-  use_fortran = False
-
 from .algebra import isnumber 
 
 def generalized_kane_mele(r1,r2,rm,fun=0.0,tol=1e-5):
-  """Return the Kane-Mele generalized Hamiltonian"""
-  if fun==0.0: return 0
-  if isnumber(fun): kmfun = lambda r: fun # function that always returns fun
-  elif callable(fun): kmfun = fun # callable function
-  else: raise # no idea
-  nsites = len(r1) # number of sites
-  mout = [[None for i in range(nsites)] for j in range(nsites)]
-  for i in range(nsites):
-    mout[i][i] = csc_matrix(np.zeros((2,2))) 
-  for i in range(nsites): # loop over initial site
-    for j in range(nsites): # loop over final site
-      dr = r1[i]-r2[j] # difference
-      if dr.dot(dr)>4.1:
-        continue # if too far away, next iteration
-      ur = km_vector(r1[i],r2[j],rm,tol=tol) # kane mele vector
-      r3 = (r1[i] + r2[j])/2.0
-      sm = (sx*ur[0] + sy*ur[1] + sz*ur[2])*kmfun(r3) # contribution
-      if mout[i][j] is None: mout[i][j] = csc_matrix(1j*sm) # add contribution
-      else: mout[i][j] += csc_matrix(1j*sm) # add contribution
-  return bmat(mout) # return matrix
+    """Return the Kane-Mele generalized Hamiltonian"""
+    if fun==0.0: return 0
+    if isnumber(fun): kmfun = lambda r: fun # function that always returns fun
+    elif callable(fun): kmfun = fun # callable function
+    else: raise # no idea
+    nsites = len(r1) # number of sites
+    mout = [[None for i in range(nsites)] for j in range(nsites)]
+    for i in range(nsites):
+        mout[i][i] = csc_matrix(np.zeros((2,2))) 
+    from .neighbor import find_first_neighbor,find_close_neighbors
+    rm = np.array(rm)
+    r1 = np.array(r1)
+    r2 = np.array(r2)
+    for i in range(nsites): # loop over initial site
+        indsim = find_close_neighbors(r1[i],rm,d=1.1) # get close enough
+        rmi = [rm[im] for im in indsim] # retain only those sites for the loop
+        indsj = find_close_neighbors(r1[i],r2,d=4.1) # get close enough
+        # this can (and should) be done more efficiently
+        for j in indsj: # loop over final site
+            dr = r1[i]-r2[j] # difference
+            if dr.dot(dr)<4.1: # if close enough
+                ur = km_vector(r1[i],r2[j],rmi,tol=tol) # kane mele vector
+                r3 = (r1[i] + r2[j])/2.0
+                sm = (sx*ur[0] + sy*ur[1] + sz*ur[2])*kmfun(r3) # contribution
+                if mout[i][j] is None: 
+                    mout[i][j] = csc_matrix(1j*sm) # add contribution
+                else: 
+                    mout[i][j] += csc_matrix(1j*sm) # add contribution
+    return bmat(mout) # return matrix
 
 
-def km_vector(ri,rj,rm,use_fortran=use_fortran,
-        tol=1e-5):
+def km_vector(ri,rj,rm,tol=1e-5):
   """Return the Kane Mele vector"""
-  if tol>1e-5: 
-      use_fortran = False
-  if use_fortran: return kanemelef90.kmvector(ri,rj,rm)
-  else:  
-      v = np.array([0.,0.,0.])
-      return km_vector_jit(ri,rj,v,np.array(rm),tol=tol)
-#      rmin = 1. -tol
-#      rmax = 1. +tol
-#      for k in range(len(rm)): # look for an intermediate site
-#        dr1 = rm[k]-ri # difference
-#        dr2 = rj-rm[k] # difference
-#        if rmin<dr1.dot(dr1)<rmax and rmin<dr2.dot(dr2)<rmax: # if connected
-#           ur = np.cross(dr1,dr2) # Kane Mele vector
-#           return ur
-#      return np.array([0.,0.,0.])
+  v = np.array([0.,0.,0.])
+  return km_vector_jit(np.array(ri).real,np.array(rj).real,v,np.array(rm).real,tol=tol)
 
 
 @jit(nopython=True)
 def km_vector_jit(ri,rj,v,rm,tol=1e-5):
-    rmin = 1. -tol
-    rmax = 1. +tol
-    for k in range(len(rm)): # look for an intermediate site
+    for k in range(rm.shape[0]): # look for an intermediate site
       dr1 = rm[k]-ri # difference
       dr2 = rj-rm[k] # difference
-      if rmin<dr1.dot(dr1)<rmax and rmin<dr2.dot(dr2)<rmax: # if connected
+      dr1m = np.sum(dr1*dr1)
+      dr2m = np.sum(dr2*dr2)
+      if abs(dr1m-1.)<tol and abs(dr2m-1.)<tol: # if connected
          v = np.cross(dr1,dr2) # Kane Mele vector
          return v
     v = np.array([0.,0.,0.])
@@ -85,22 +73,20 @@ def haldane(r1,r2,rm,fun=0.0,sublattice=None):
         from .potentials import array2potential
         kmfun = array2potential(r1[:,0],r1[:,1],fun)
     nsites = len(r1) # number of sites
-    mout = np.zeros((nsites,nsites),dtype=np.complex) # initialize
+    mout = np.zeros((nsites,nsites),dtype=np.complex128) # initialize
     from . import neighbor
     neighs = neighbor.connections(r1,rm) # list with neighbors of each site
     for i in range(nsites): # loop over initial site
       rijs = [rm[kk] for kk in neighs[i]] # loop over first neighbors
       if len(rijs)==0: continue
-      for j in range(nsites): # loop over final site
+      indsj = neighbor.find_close_neighbors(r1[i],r2,d=1.9) # get close enough
+      for j in indsj: # loop over final site
         dr = r1[i]-r2[j] # difference
-        if dr.dot(dr)>4.1:
-          continue # if too far away, next iteration
-  #      ur = km_vector(r1[i],r2[j],rm) # kane mele vector
-  #      print(rijs)
-        ur = km_vector(r1[i],r2[j],rijs) # kane mele vector
-        r3 = (r1[i] + r2[j])/2.0
-        sm = ur[2]*kmfun(r3) # clockwise or anticlockwise
-        mout[i,j] = 1j*sm*(sublattice[i]+sublattice[j])/2. # store
+        if dr.dot(dr)<4.1:
+            ur = km_vector(r1[i],r2[j],rijs) # kane mele vector
+            r3 = (r1[i] + r2[j])/2.0
+            sm = ur[2]*kmfun(r3) # clockwise or anticlockwise
+            mout[i,j] = 1j*sm*(sublattice[i]+sublattice[j])/2. # store
     return csc_matrix(mout) # return matrix
 
 
