@@ -10,6 +10,8 @@ from . import kpm
 from . import sculpt
 from . import parallel
 from . import algebra
+from . import operators
+from .algebra import dagger
 
 def write_kdos(k=0.,es=[],ds=[],new=True):
   """ Write KDOS in a file"""
@@ -149,53 +151,57 @@ def write_surface_3d(h,energies=None,klist=None,delta=0.01):
 
 def kdos_bands(h,use_kpm=False,kpath=None,scale=10.0,frand=None,
                  P = None,
-                 ewindow=4.0,ne=1000,delta=0.01,ntries=10,nk=100,
+                 ewindow=4.0,delta=0.01,ntries=10,nk=100,
                  operator=None,energies=np.linspace(-3.0,3.0,200),
                  mode="ED",**kwargs):
-  """Calculate the KDOS bands using the KPM"""
-  if use_kpm: mode ="KPM" # conventional method
-  if mode=="ED":
-      from . import dos
-      def pfun(k):
-        (es,ds) = h.get_dos(ks=[k],operator=operator,energies=energies,
-                delta=delta)
-        return energies,ds
-  elif mode=="green":
-    f = h.get_gk_gen(delta=delta) # Green generator
-    def pfun(k): # do it for this k-point
-        def gfun(e):
-            m = f(k=k,e=e) # Green's function
-            m = green.GtimesO(m,operator,k=k)
-            return -algebra.trace(m).imag # return DOS
-        return energies,np.array([gfun(e) for e in energies])
-  elif mode=="KPM": # KPM method
-    if operator is not None: 
-        from .operators import Operator
-        operator = Operator(operator).get_matrix() # convert to matrix
-    hkgen = h.get_hk_gen() # get generator
-    def pfun(k): # do it for this k-point
-      print("Doing",k)
-      hk = hkgen(k) # get Hamiltonian
-      npol = int(scale/delta) # number of polynomials
-      (x,y) = kpm.pdos(hk,scale=scale,npol=npol,ne=ne,P=P,
-                   operator=operator,
-                   ewindow=ewindow,ntries=ntries,x=energies) # compute
-      return (x,y)
-  if kpath is None:
-      kpath = h.geometry.get_kpath(kpath,nk=nk) # generate kpath
-  ### Now compute and write in a file
-  ik = 0
-  out = parallel.pcall(pfun,kpath) # compute all
-  fo = open("KDOS_BANDS.OUT","w") # open file
-  for k in kpath: # loop over kpoints
-    (x,y) = out[ik] # get this one
-    for (ix,iy) in zip(x,y): # loop
-      fo.write(str(ik/len(kpath))+"   ")
-      fo.write(str(ix)+"   ")
-      fo.write(str(iy)+"\n")
-    fo.flush()
-    ik += 1
-  fo.close()
+    """Calculate the KDOS bands using the KPM"""
+    if use_kpm: mode ="KPM" # conventional method
+    if mode=="ED":
+        from . import dos
+        def pfun(k):
+          (es,ds) = h.get_dos(ks=[k],operator=operator,energies=energies,
+                  delta=delta,**kwargs)
+          return energies,ds
+    elif mode=="green":
+      f = h.get_gk_gen(delta=delta) # Green generator
+      def pfun(k): # do it for this k-point
+          def gfun(e):
+              m = f(k=k,e=e) # Green's function
+              m = green.GtimesO(m,operator,k=k)
+              return -algebra.trace(m).imag # return DOS
+          return energies,np.array([gfun(e) for e in energies])
+    elif mode=="KPM": # KPM method
+      if operator is not None: 
+          from .operators import Operator
+          operator = Operator(operator).get_matrix() # convert to matrix
+      h = h.copy()
+      h.turn_sparse()
+      hkgen = h.get_hk_gen() # get generator
+      def pfun(k): # do it for this k-point
+#        print("Doing",k)
+        hk = hkgen(k) # get Hamiltonian
+        npol = 3*int(scale/delta) # number of polynomials
+        (x,y) = kpm.pdos(hk,scale=scale,npol=npol,ne=npol*4,P=P,
+                     operator=operator,
+                     ewindow=ewindow,ntries=ntries,x=energies,
+                     **kwargs) # compute
+        return (x,y)
+    if kpath is None:
+        kpath = h.geometry.get_kpath(kpath,nk=nk) # generate kpath
+    ### Now compute and write in a file
+    ik = 0
+    out = parallel.pcall(pfun,kpath) # compute all
+    fo = open("KDOS_BANDS.OUT","w") # open file
+    for k in kpath: # loop over kpoints
+      (x,y) = out[ik] # get this one
+      for (ix,iy) in zip(x,y): # loop
+        fo.write(str(ik/len(kpath))+"   ")
+        fo.write(str(ix)+"   ")
+        fo.write(str(iy)+"\n")
+      fo.flush()
+      ik += 1
+    fo.close()
+    return np.genfromtxt("KDOS_BANDS.OUT").T
 
 
 
@@ -227,7 +233,7 @@ def write_surface_kpm(h,ne=400,klist=None,scale=4.,npol=200,w=20,ntries=20):
       def gedge(): return (np.random.random(len(inde))-0.5)*inde
       def gbulk(): return (np.random.random(len(indb))-0.5)*(indb)
       # hamiltonian
-      h0 = intra + inter*np.exp(1j*np.pi*2.*k) + (inter*np.exp(1j*np.pi*2.*k)).H
+      h0 = intra + inter*np.exp(1j*np.pi*2.*k) + dagger(inter*np.exp(1j*np.pi*2.*k))
       xs = np.linspace(-0.9,0.9,4*npol) # x points
       es = xs*scale
       # calculate the bulk
@@ -303,7 +309,8 @@ def interface(h1,h2,energies=np.linspace(-1.,1.,100),operator=None,
 
 def surface_kdos(h1,energies=np.linspace(-1.,1.,100),operator=None,
                     delta=0.01,kpath=None,hs=None,nsuper=None,
-                    write=True,nk=None):
+                    info = False,
+                    write=True,nk=None,**kwargs):
     """Get the surface DOS of an interface"""
     if nk is None: nk = len(energies)
     h1 = h1.get_supercell(nsuper)
@@ -319,7 +326,7 @@ def surface_kdos(h1,energies=np.linspace(-1.,1.,100),operator=None,
         else: raise
     if write: fo = open("KDOS.OUT","w")
     if write: fo.write("# k, E, Surface, Bulk\n")
-    tr = timing.Testimator("KDOS") # generate object
+    if info: tr = timing.Testimator("KDOS") # generate object
     ik = 0
     h1 = h1.get_multicell() # multicell Hamiltonian
     kout = [] # storage
@@ -327,10 +334,10 @@ def surface_kdos(h1,energies=np.linspace(-1.,1.,100),operator=None,
     dsout = [] # storage
     dbout = [] # storage
     for k in kpath:
-      tr.remaining(ik,len(kpath)) # generate object
+      if info: tr.remaining(ik,len(kpath)) # generate object
       ik += 1
       outs = green.surface_multienergy(h1,k=k,energies=energies,
-                           delta=delta,hs=hs)
+                           delta=delta,hs=hs,**kwargs)
       for (energy,out) in zip(energies,outs):
         # write everything
         kout.append(k[0]) # add to the output
@@ -341,6 +348,8 @@ def surface_kdos(h1,energies=np.linspace(-1.,1.,100),operator=None,
         do = []
         for g in out: # loop
           if operator is None: d = -algebra.trace(g).imag # only the trace 
+          elif type(operator)==operators.Operator:
+              d = -(operators.Operator(g)*operator).trace().imag
           elif callable(operator): d = operator(g,k=k) # call the operator
           else:  d = -algebra.trace(g@operator).imag # assume it is a matrix
           if write: fo.write(str(d)+"   ") # write in a file
