@@ -3,7 +3,7 @@
 import numpy as np
 import scipy.linalg as lg
 from . import geometry
-
+from numba import jit
 
 def get_klist(g,ns,nk=100):
     """Return a klist from a list of names"""
@@ -300,24 +300,25 @@ def infer_kmesh_density(kmesh,d=2):
 
 
 
-def kmesh(dimensionality,nk=10,nsuper=1):
+def kmesh(dimensionality,nk=10,nsuper=1,
+        endpoint=False):
   """Return a mesh of k-points for a certain dimensionality"""
   from .checkclass import number2array
   kp = []
   if nk==1 or dimensionality==0: return [[0.,0.,0.]]
   if dimensionality==1:
-    for k1 in np.linspace(0.,nsuper,nk,endpoint=False):
+    for k1 in np.linspace(0.,nsuper,nk,endpoint=endpoint):
       kp.append([k1,0.,0.]) # store
   elif dimensionality==2:
     nk = number2array(nk) # return an array
-    for k1 in np.linspace(0.,nsuper,nk[0],endpoint=False):
-      for k2 in np.linspace(0.,nsuper,nk[1],endpoint=False):
+    for k1 in np.linspace(0.,nsuper,nk[0],endpoint=endpoint):
+      for k2 in np.linspace(0.,nsuper,nk[1],endpoint=endpoint):
         kp.append([k1,k2,0.]) # store
   elif dimensionality==3:
     nk = number2array(nk) # return an array
-    for k1 in np.linspace(0.,nsuper,nk[0],endpoint=False):
-      for k2 in np.linspace(0.,nsuper,nk[1],endpoint=False):
-        for k3 in np.linspace(0.,nsuper,nk[2],endpoint=False):
+    for k1 in np.linspace(0.,nsuper,nk[0],endpoint=endpoint):
+      for k2 in np.linspace(0.,nsuper,nk[1],endpoint=endpoint):
+        for k3 in np.linspace(0.,nsuper,nk[2],endpoint=endpoint):
           kp.append([k1,k2,k3]) # store
   else: raise
   kp = [np.array(k) for k in kp] # to array
@@ -348,5 +349,21 @@ def get_kpath(g,kpath=None,**kwargs):
     else: return np.array(kpath) # assume is a valid list of vectors
 
 
+@jit(nopython=True)
+def kgrid2d(kxs,kys):
+    """Return a list of kvector that create a grid over the inputs"""
+    nx = len(kxs)
+    ny = len(kys)
+    out = np.zeros((nx*ny,3),dtype=float) # create array
+    iz = 0 # initialize
+    for ix in range(nx):
+        x = kxs[ix] # x coordinate
+        for iy in range(ny):
+            y = kys[iy] # x coordinate
+            out[iz,0] = x # store
+            out[iz,1] = y # store
+            iz += 1 # increase counter
+    return out
+
 
 
