@@ -19,7 +19,9 @@ def label2k(g,kl):
     elif kl=="M3": return [.5,.5,.0]
     elif kl=="X": return [.5,.0,.0]
     elif kl=="Y": return [.0,.5,.0]
-    else: raise
+    else: 
+        print("Unrecognized kpoint",kl)
+        raise
 
 
 
