@@ -14,12 +14,13 @@ from . import parallel
 from . import algebra
 from .increase_hilbert import full2profile as spatial_dos
 from . import filesystem as fs
+from .algebra import dagger
 
 def ldos0d(h,e=0.0,delta=0.01,write=True):
   """Calculates the local density of states of a Hamiltonian and
      writes it in file"""
   if h.dimensionality==0:  # only for 0d
-    iden = np.identity(h.intra.shape[0],dtype=np.complex) # create identity
+    iden = np.identity(h.intra.shape[0],dtype=np.complex128) # create identity
     g = ( (e+1j*delta)*iden -h.intra ).I # calculate green function
   else: raise # not implemented...
   d = [ -(g[i,i]).imag/np.pi for i in range(len(g))] # get imaginary part
@@ -138,53 +139,9 @@ def ldos_arpack(intra,num_wf=10,robust=False,tol=0,e=0.0,delta=0.01):
 
 
 
-def ldos_waves(intra,es = [0.0],delta=0.01,operator=None,
-        num_bands=None,k=None,delta_discard=None,**kwargs):
-  """Calculate the DOS in a set of energies by full diagonalization"""
-  es = np.array(es) # array with energies
-  if num_bands is None:
-      eig,eigvec = algebra.eigh(intra) 
-  else:
-      eig,eigvec = algebra.smalleig(intra,numw=num_bands,evecs=True)
-      eigvec = eigvec.T
-  ds = [] # empty list
-  if operator is None: weights = eig*0. + 1.0
-  else: weights = [operator.braket(v,k=k) for v in eigvec.transpose()] # weights
-  if delta_discard is not None: # discard too far values
-      ewin = [min(es)-delta_discard*delta,min(es)+delta_discard*delta]
-      for i in range(len(weights)):
-          e = eig[i]
-          if not ewin[0]<e<ewin[1]: weights[i] = 0.0
-  v2s = [(np.conjugate(v)*v).real for v in eigvec.transpose()]
-  ds = [[0.0 for i in range(intra.shape[0])] for e in es] # initialize
-  ds = ldos_waves_jit(np.array(es),
-          np.array(eigvec).T,np.array(eig),np.array(weights),
-          np.array(v2s),np.array(ds),delta)
-  return ds
 
-
-
-@jit(nopython=True)
-def ldos_waves_jit(es,eigvec,eig,weights,v2s,ds,delta):
-  for i in range(len(es)): # loop over energies
-    energy = es[i] # energy
-    d = ds[i]
-    for j in range(len(eig)):
-        v = eigvec[j]
-        ie = eig[j]
-        weight = weights[j]
-        v2 = v2s[j]
-        fac = delta/((energy-ie)**2 + delta**2) # factor to create a delta
-        d += weight*fac*v2 # add contribution
-    d /= np.pi # normalize
-    ds[i] = d # store
-  return ds
-
-
-def ldos_diagonalization(m,e=0.0,**kwargs):
-    """Compute the LDOS using exact diagonalization"""
-#    if algebra.issparse(m): return ldos_arpack(m,e=e,**kwargs) # sparse
-    return ldos_waves(m,es=[e],**kwargs)[0] # dense
+from .ldostk.ldoswaves import ldos_diagonalization
+from .ldostk.ldoswaves import ldos_waves
 
 
 
@@ -263,7 +220,7 @@ def ldos_projector(h,e=0.0,**kwargs):
     inds = np.array(range(len(d))) # indexes
     n = len(d)
     d = d/np.sum(d) # normalize
-    m = csc_matrix((d,(inds,inds)),shape=(n,n),dtype=np.complex) # matrix
+    m = csc_matrix((d,(inds,inds)),shape=(n,n),dtype=np.complex128) # matrix
     m = h.spinless2full(m) # to full matrix
     return operators.Operator(m) # convert to operator
 
@@ -279,10 +236,17 @@ def ldos_potential(h,**kwargs):
     return # not finished yet
 
 
+def get_ldos(h,**kwargs):
+    """Master method for LDOS"""
+    if not h.non_hermitian: # Hermitian case
+        return get_ldos_general(h,**kwargs)
+    else:
+        from .nonhermitiantk.nhmethods import get_ldos as get_ldos_NH
+        return get_ldos_NH(h,**kwargs)
 
 
-def get_ldos(h,projection="TB",**kwargs):
-    """ Calculate LDOS"""
+def get_ldos_general(h,projection="TB",**kwargs):
+    """ Calculate LDOS with different methods"""
     if projection=="TB": return get_ldos_tb(h,**kwargs)
     elif projection=="TBRS": return get_ldos_tb(h,interpolate=True,**kwargs)
     elif projection=="atomic": 
@@ -293,14 +257,16 @@ def get_ldos(h,projection="TB",**kwargs):
 
 def get_ldos_tb(h,e=0.0,delta=0.001,nrep=5,nk=None,ks=None,mode="arpack",
              random=False,silent=True,interpolate=False,
+             operator=None,return_rd = False,
              write=True,**kwargs):
     """ Calculate LDOS in a tight binding basis"""
     if ks is not None and mode=="green": raise
+    if operator is not None: operator = h.get_operator(operator)
     if mode=="green":
       from . import green
       if h.dimensionality!=2: raise # only for 2d
       h = h.copy()
-      h.turn_dense()
+      h = h.get_dense()
       if nk is not None:
         print("LDOS using normal integration with nkpoints",nk)
         gb,gs = green.bloch_selfenergy(h,energy=e,delta=delta,mode="full",nk=nk)
@@ -327,20 +293,32 @@ def get_ldos_tb(h,e=0.0,delta=0.001,nrep=5,nk=None,ks=None,mode="arpack",
     # write result
     d = spatial_dos(h,d) # convert to spatial resolved DOS
     g = h.geometry  # store geometry
-    x,y = g.x,g.y # get the coordinates
+    x,y,z,r = g.x,g.y,g.z,g.r # get the coordinates
     if nrep>1:
         go = h.geometry.copy() # copy geometry
         go = go.supercell(nrep) # create supercell
         do = d.tolist()*(nrep**g.dimensionality) # replicate
         xo = go.x
         yo = go.y
-    else: xo,yo,do = x,y,d
+        zo = go.z
+        ro = go.r
+    else: xo,yo,zo,ro,do = x,y,z,r,d
     if interpolate:
         from .interpolation import atomic_interpolation
         xo,yo,do = atomic_interpolation(xo,yo,do,**kwargs)
+        if return_rd: raise # not implemented
     if write: 
+#        if return_rd: raise # not implemented
         write_ldos(xo,yo,do) # write in file
-    return (xo,yo,do) # return LDOS
+    xo = np.array(xo)
+    yo = np.array(yo)
+    zo = np.array(zo)
+    ro = np.array(ro)
+    do = np.array(do)
+    if return_rd: # return positions and DOS
+        return (ro,do) # return LDOS
+    else: # return x,y and DOS
+        return (xo,yo,do) # return LDOS
 
 
 ldos = get_ldos # for backcompatibility
@@ -372,7 +350,7 @@ def multi_ldos_tb(h,es=np.linspace(-1.0,1.0,100),delta=0.01,
       if random:
         k = np.random.random(3) # random vector
         print("RANDOM vector in LDOS")
-      e,w = smalleig(hk(k),numw=num_bands,evecs=True)
+      e,w = smalleig(hk(k),numw=num_bands,evecs=True,e0=np.mean(es))
       evals += [ie for ie in e]
       ws += [iw for iw in w]
       ps += [op(iw,k=k) for iw in w] # weights
@@ -455,7 +433,7 @@ def ldos_finite(h,e=0.0,n=10,nwf=4,delta=0.0001):
   if h.dimensionality!=1: raise # if it is not one dimensional
   intra = csc(h.intra) # convert to sparse
   inter = csc(h.inter) # convert to sparse
-  interH = inter.H # hermitian
+  interH = dagger(inter) # hermitian
   m = [[None for i in range(n)] for j in range(n)] # full matrix
   for i in range(n): # add intracell
     m[i][i] = intra
