@@ -18,11 +18,12 @@ def ldos_generator(h,**kwargs):
 
 
 def profile_generator(h,delta=0.05,nrep=1,nk=20,dl=None,mode="LDOS",
-        ratomic=1.5,dr=0.2,num_bands=None,**kwargs):
+        ratomic=1.5,dr=0.2,num_bands=None,
+        deltax=None, deltay=None,**kwargs):
     """Compute the LDOS at different eenrgies, and add an envelop atomic
     orbital"""
     h = h.copy() # copy the Hamiltonian
-    h.turn_dense() # dense hamiltonian
+    h = h.get_dense() # dense hamiltonian
     evals,vs,ks = h.get_eigenvectors(nk=nk,kpoints=True,
             numw=num_bands) # compute wavefunctions
     if dl is None: 
@@ -34,8 +35,10 @@ def profile_generator(h,delta=0.05,nrep=1,nk=20,dl=None,mode="LDOS",
     ##########################################################
     lodict = dict() # dictionary for the local orbitals
     # get the grids
+    if deltax is None: deltax=ratomic
+    if deltay is None: deltay=ratomic
     x,y = get_grids(h.geometry,nrep=nrep,dr=dr,
-            deltax=ratomic,deltay=ratomic)
+            deltax=deltax,deltay=deltay)
     r = np.zeros((len(x),3)) ; r[:,0] = x ; r[:,1] = y
     # now chack which centers to accept
     xmin,xmax = np.min(x),np.max(x)
@@ -136,7 +139,7 @@ def get_real_space_density_generator(lodict,g,has_spin=False):
         """Function that return the real space density"""
         nc = len(w) # number of components of the Bloch wavefunction
         phis = np.array([g.bloch_phase(d,k) for d in ds]) # phases
-        out = np.zeros(orbs[0].shape[0],dtype=np.complex)
+        out = np.zeros(orbs[0].shape[0],dtype=np.complex128)
         if not has_spin: # spinless
           return get_real_space_density_jit(w,phis,inds,orbs,out).real
         else: # spinful
