@@ -1,6 +1,8 @@
 import numpy as np
 from scipy.sparse import csc_matrix,bmat,coo_matrix
 from . import parallel
+from .algebra import dagger
+from numba import jit
 
 
 def collect_hopping(h):
@@ -22,8 +24,11 @@ def collect_hopping(h):
 
 class Hopping(): 
   def __init__(self,d=None,m=None):
-    self.dir = np.array(d)
-    self.m = m
+      self.dir = np.array(d)
+      self.m = m
+  def copy(self):
+      from copy import deepcopy
+      return deepcopy(self)
 
 
 def turn_multicell(h):
@@ -47,7 +52,8 @@ def turn_multicell(h):
       del ho.ty
       del ho.txy
       del ho.txmy
-    else: raise
+    else: 
+        return h # 3d is always multicell
     dd = dict() # dictionary
     dd[(0,0,0)] = h.intra
     for (d,t) in zip(dirs,ts): 
@@ -65,24 +71,22 @@ def set_multihopping(ho,dd):
 
 def set_dictionary(ho,dd):
     """Set a dictionary as the Hamiltonian"""
-    ho.intra = dd[(0,0,0)] # intracell
+    ho.intra = dd[(0,0,0)].copy() # intracell
     hoppings = [] # list of hoppings
     for d in dd: # loop over hoppings
         t = dd[d] # matrix
         if d==(0,0,0): continue
         hopping = Hopping() # create object
-        hopping.m = t
+        hopping.m = t.copy()
         hopping.dir = np.array(d)
         hoppings.append(hopping) # store
-#        hopping = Hopping() # create object
-#        hopping.m = np.conjugate(t).T
-#        hopping.dir = -np.array(d)
-#        hoppings.append(hopping) # store
     ho.hopping = hoppings # store all the hoppings
     ho.is_multicell = True # multicell hamiltonian
     return ho
 
 
+
+
 def generate_get_tij(h):
     """Get the hopping between cells with certain indexes"""
     hdict = h.get_multihopping().get_dict() # generate the multihopping object
@@ -96,45 +100,52 @@ def generate_get_tij(h):
     return fun
 
 def hk_gen(h):
-  """Generate a k dependent hamiltonian"""
-  if not h.is_multicell:
-      h = h.get_multicell()
-  # get the non zero hoppings
-  hopping = [] # empty list
-  for t in h.hopping: # loop
-    if h.is_sparse:
-      if np.sum(np.abs(coo_matrix(t.m).data))>1e-7: hopping.append(t) # store this hopping
-    else:
-      if np.sum(np.abs(t.m))>1e-7: hopping.append(t) # store this hopping
-  if h.dimensionality == 0: return lambda k: h.intra
-  elif h.dimensionality == 1: # one dimensional
-    def hk(k):
-      """k dependent hamiltonian, k goes from 0 to 1"""
-      mout = h.intra.copy() # intracell term
-      for t in hopping: # loop over matrices
-        tk = t.m * h.geometry.bloch_phase(t.dir,k) # k hopping
-        mout = mout + tk 
-      return mout
-    return hk  # return the function
-  elif h.dimensionality == 2: # two dimensional
-    def hk(k):
-      """k dependent hamiltonian, k goes from 0 to 1"""
-      mout = h.intra.copy() # intracell term
-      for t in hopping: # loop over matrices
-        tk = t.m * h.geometry.bloch_phase(t.dir,k) # k hopping
-        mout = mout + tk 
-      return mout
-    return hk  # return the function
-  elif h.dimensionality == 3: # three dimensional
-    def hk(k):
-      """k dependent hamiltonian, k goes from 0 to 1"""
-      mout = h.intra.copy() # intracell term
-      for t in h.hopping: # loop over matrices
-        tk = t.m * h.geometry.bloch_phase(t.dir,k) # k hopping
-        mout = mout + tk 
-      return mout
-    return hk  # return the function
-  else: raise
+    """Generate a k dependent hamiltonian"""
+    if not h.is_multicell:
+        h = h.get_multicell()
+    # get the non zero hoppings
+    hopping = [] # empty list
+    for t in h.hopping: # loop
+      if h.is_sparse:
+        if np.sum(np.abs(coo_matrix(t.m).data))>1e-7: hopping.append(t) # store this hopping
+      else:
+        if np.sum(np.abs(t.m))>1e-7: hopping.append(t) # store this hopping
+    ## dense Hamiltonians, accelerated function ##
+    if not h.is_sparse: # for dense Hamiltonians
+        from .htk.bloch import bloch_hamiltonian_generator_dense
+        return bloch_hamiltonian_generator_dense(h,hopping)
+    ## sparse Hamiltonians, explicit function ##
+    else: # sparse Hamiltonians
+        if h.dimensionality == 0: 
+            return lambda k: h.intra
+        elif h.dimensionality == 1: # one dimensional
+          def hk(k):
+            """k dependent hamiltonian, k goes from 0 to 1"""
+            mout = h.intra.copy() # intracell term
+            for t in hopping: # loop over matrices
+              tk = t.m * h.geometry.bloch_phase(t.dir,k) # k hopping
+              mout = mout + tk 
+            return mout
+          return hk  # return the function
+        elif h.dimensionality == 2: # two dimensional
+          def hk(k):
+            """k dependent hamiltonian, k goes from 0 to 1"""
+            mout = h.intra.copy() # intracell term
+            for t in hopping: # loop over matrices
+              tk = t.m * h.geometry.bloch_phase(t.dir,k) # k hopping
+              mout = mout + tk 
+            return mout
+          return hk  # return the function
+        elif h.dimensionality == 3: # three dimensional
+          def hk(k):
+            """k dependent hamiltonian, k goes from 0 to 1"""
+            mout = h.intra.copy() # intracell term
+            for t in h.hopping: # loop over matrices
+              tk = t.m * h.geometry.bloch_phase(t.dir,k) # k hopping
+              mout = mout + tk 
+            return mout
+          return hk  # return the function
+        else: raise
 
 
 
@@ -156,51 +167,8 @@ def turn_spinful(h,enforce_tr=False):
     h.hopping[i].m = fun(h.hopping[i].m) # spinful hopping
 
 
-
-
-def bulk2ribbon(hin,n=10,sparse=True,nxt=6,ncut=6):
-  """ Create a ribbon hamiltonian object"""
-  if not hin.is_multicell: h = turn_multicell(hin)
-  else: h = hin # nothing othrwise
-  hr = h.copy() # copy hamiltonian
-  if sparse: hr.is_sparse = True # sparse output
-  hr.dimensionality = 1 # reduce dimensionality
-  # stuff about geometry
-  hr.geometry = h.geometry.supercell((1,n)) # create supercell
-  hr.geometry.dimensionality = 1
-  hr.geometry.a1 = h.geometry.a1 # add the unit cell vector
-  from . import sculpt # rotate the geometry
-  hr.geometry = sculpt.rotate_a2b(hr.geometry,hr.geometry.a1,np.array([1.,0.,0.]))
-  hr.geometry.celldis = hr.geometry.a1[0]
-  get_tij = generate_get_tij(h) # return a function to abtain the hoppings
-  def superhopping(dr=[0,0,0]): 
-    """ Return a matrix with the hopping of the supercell"""
-    intra = [[None for i in range(n)] for j in range(n)] # intracell term
-    for ii in range(n): # loop over ii
-      for jj in range(n): # loop over jj
-        d = np.array([dr[0],ii-jj+dr[1],dr[2]])
-        if d.dot(d)>ncut*ncut: continue # skip iteration
-        m = get_tij(rij=d) # get the matrix
-        if m is not None: intra[ii][jj] = csc_matrix(m) # store
-        else: 
-          if ii==jj: intra[ii][jj] = csc_matrix(h.intra*0.)
-    intra = bmat(intra) # convert to matrix
-    if not sparse: intra = intra.todense() # dense matrix
-    return intra
-  # get the intra matrix
-  hr.intra = superhopping()
-  # now do the same for the interterm
-  hoppings = [] # list of hopings
-  for i in range(-nxt,nxt+1): # loop over hoppings
-    if i==0: continue # skip the 0
-    d = np.array([i,0.,0.])
-    hopp = Hopping() # create object
-    hopp.m = superhopping(dr=d) # get hopping of the supercell
-    hopp.dir = d
-    hoppings.append(hopp)
-  hr.hopping = hoppings # store the list
-  hr.dimensionality = 1
-  return hr 
+from .htk.supercell import bulk2ribbon
+from .htk.supercell import bulk2film
 
 
 
@@ -236,14 +204,14 @@ def basis_change(h,R):
   R = np.matrix(R) # convert to matrix
   for i in range(len(h.hopping)):
     m = h.hopping[i].m # rotation of the direction
-    ho.hopping[i].m = R.H*m*R # Hamiltonian in new basis
+    ho.hopping[i].m = dagger(R)@m@R # Hamiltonian in new basis
   return ho
 
 
 
 
 
-def clean(h,cutoff=0.0001):
+def clean(h,cutoff=1e-4):
   """Remove hoppings smaller than a certain quantity"""
   ho = h.copy() # copy hamiltonian
   raise 
@@ -267,14 +235,14 @@ def supercell_hamiltonian(hin,nsuper=[1,1,1],sparse=True,ncut=3,
     for j in range(nsuper[1]):
       for k in range(nsuper[2]):
         pos.append(np.array([i,j,k])) # store position inside the supercell
-  zero = csc_matrix(np.zeros(h.intra.shape,dtype=np.complex)) # zero matrix
-  get_tij = generate_get_tij(h) # return a function to abtain the hoppings
+  zero = csc_matrix(np.zeros(h.intra.shape,dtype=np.complex128)) # zero matrix
+  get_tij = generate_get_tij(h) # return a function to obtain the hoppings
   def superhopping(dr=[0,0,0]): 
     """ Return a matrix with the hopping of the supercell"""
     rs = [dr[0]*nsuper[0],dr[1]*nsuper[1],dr[2]*nsuper[2]] # supercell vector
     intra = [[None for i in range(n)] for j in range(n)] # intracell term
-    for ii in range(n): intra[ii][ii] = zero.copy() # zero
-
+    for ii in range(n): 
+        intra[ii][ii] = zero.copy() # zero
     for ii in range(n): # loop over cells
       for jj in range(n): # loop over cells
         d = pos[jj] + np.array(rs) -pos[ii] # distance
@@ -289,15 +257,19 @@ def supercell_hamiltonian(hin,nsuper=[1,1,1],sparse=True,ncut=3,
   hr.intra = superhopping()
   # now do the same for the interterm
   hoppings = [] # list of hopings
-  for i in range(-ncut,ncut+1): # loop over hoppings
-    for j in range(-ncut,ncut+1): # loop over hoppings
-      for k in range(-ncut,ncut+1): # loop over hoppings
+  nxs,nys,nzs = [0],[0],[0] # initialize
+  if hin.dimensionality>0:  nxs = range(-ncut,ncut+1)
+  if hin.dimensionality>1:  nys = range(-ncut,ncut+1)
+  if hin.dimensionality>2:  nzs = range(-ncut,ncut+1)
+  for i in nxs: # loop over hoppings
+    for j in nys: # loop over hoppings
+      for k in nzs: # loop over hoppings
         if i==j==k==0: continue # skip the intraterm
         dr = np.array([i,j,k]) # set as array
         hopp = Hopping() # create object
         hopp.m = superhopping(dr=dr) # get hopping of the supercell
         hopp.dir = dr
-        if np.sum(np.abs(hopp.m))>0.00000001: # skip this matrix
+        if np.sum(np.abs(hopp.m))>1e-6: # skip this matrix
           hoppings.append(hopp)
         else: pass
       hr.hopping = hoppings # store the list
@@ -395,7 +367,7 @@ def read_from_file(input_file="hamiltonian.wan"):
   tlist = []
   norb = np.max([np.max(np.abs(m[3])),np.max(np.abs(m[4]))])
   norb = int(norb)
-  zeros = np.matrix(np.zeros((norb,norb),dtype=np.complex)) # zero matrix
+  zeros = np.matrix(np.zeros((norb,norb),dtype=np.complex128)) # zero matrix
   def get_t(i,j,k):
     mo = zeros.copy() # copy matrix
     found = False
@@ -422,7 +394,7 @@ def read_from_file(input_file="hamiltonian.wan"):
   for i in range(-ncells[0],ncells[0]+1):
     for j in range(-ncells[1],ncells[1]+1):
       for k in range(-ncells[2],ncells[2]+1):
-        dm = get_hopping(i,j,k) - get_hopping(-i,-j,-k).H
+        dm = get_hopping(i,j,k) - dagger(get_hopping(-i,-j,-k))
         if np.sum(np.abs(dm))>0.0001: raise
   print("Hopping generator is Hermitian")
   return get_hopping # return the function
@@ -452,22 +424,26 @@ def pairs2hopping(ps):
 
 
 
+@jit(nopython=True)
 def close_enough(rs1,rs2,rcut=2.0):
-  """CHeck if two sets of positions are at a distance
-  at least rcut"""
-  rcut2 = rcut*rcut # square of the distance
-  for ri in rs1:
-    for rj in rs2:
-      dr = ri - rj # vector
-      dr = dr.dot(dr) # distance
-      if dr<rcut2: return True
-  return False
+    """Check if two sets of positions are at a distance
+    at least rcut"""
+    rcut2 = rcut*rcut # square of the distance
+    for ri in rs1:
+      for rj in rs2:
+        dr = ri - rj # vector
+        dr = dr.dot(dr) # distance
+        if dr<rcut2: return True
+    return False
 
 
 
-def turn_no_multicell(h):
+def turn_no_multicell(h,tol=1e-5):
   """Converts a Hamiltonian into the non multicell form"""
+  from .htk.kchain import detect_longest_hopping
   if not h.is_multicell: return h # Hamiltonian is already fine
+  if detect_longest_hopping(h)>1: raise # error
+  if h.dimensionality>2: return h # too high dimensionality
   ho = h.copy() # copy Hamiltonian
   ho.is_multicell = False
   if ho.dimensionality==0: pass
@@ -478,13 +454,14 @@ def turn_no_multicell(h):
     ho.txy = ho.intra*0.
     ho.txmy = ho.intra*0.
     ho.ty = ho.intra*0.
-  else: raise
+  else: 
+      raise # just in case
   for t in h.hopping: # loop over hoppings
     if h.dimensionality==0: pass # one dimensional
     elif h.dimensionality==1: # one dimensional
       if t.dir[0]==1 and t.dir[1]==0 and t.dir[2]==0: # 
         ho.inter = t.m # store
-      elif np.sum(np.abs(t.m))>0.0001 and np.max(np.abs(t.dir))>1: raise # Uppps, not possible
+      elif np.sum(np.abs(t.m))>tol and np.max(np.abs(t.dir))>1: raise # Uppps, not possible
     elif h.dimensionality==2: # two dimensional
       if t.dir[0]==1 and t.dir[1]==0 and t.dir[2]==0: # 
         ho.tx = t.m # store
@@ -494,43 +471,53 @@ def turn_no_multicell(h):
         ho.txy = t.m # store
       elif t.dir[0]==1 and t.dir[1]==-1 and t.dir[2]==0: # 
         ho.txmy = t.m # store
-      elif np.sum(np.abs(t.m))>0.0001 and np.max(np.abs(t.dir))>1: raise # Uppps, not possible
-    else: raise
+      elif np.sum(np.abs(t.m))>tol and np.max(np.abs(t.dir))>1: raise # Uppps, not possible
+    else: 
+        raise # just in case
   ho.hopping = [] # empty list
   return ho
 
 
 
-def kchain(h,k=[0.,0.,0.]):
-  """Return the onsite and hopping for a particular k"""
-  if not h.is_multicell: h = h.get_multicell()
-  dim = h.dimensionality # dimensionality
-  if dim==1: # 1D
-      for t in h.hopping:
-          if t.dir[0]==1: return h.intra,t.m
-      raise
-  elif dim>1: # 2D or 3D
-    intra = np.zeros(h.intra.shape) # zero amtrix
-    inter = np.zeros(h.intra.shape) # zero amtrix
-    intra = h.intra # initialize
-    for t in h.hopping: # loop over hoppings
-      tk = t.m * h.geometry.bloch_phase(t.dir,k) # k hopping
-      if t.dir[dim-1]==0: intra = intra + tk # add contribution 
-      if t.dir[dim-1]==1: inter = inter + tk # add contribution 
-    return intra,inter 
-  else: raise
-
+from .htk.kchain import kchain
 
+#
+#def kchain(h,k=[0.,0.,0.]):
+#    """Return the onsite and hopping for a particular k"""
+#    if not h.is_multicell: h = h.get_multicell()
+#    # make a check that only NN matters
+#    try:
+#        hnn = h.get_no_multicell() # no multicell Hamiltonian
+#    except:
+#        print("Hopping beyong NN unit cells in kchain, stopping")
+#        exit()
+#    dim = h.dimensionality # dimensionality
+#    if dim==1: # 1D
+#        for t in h.hopping:
+#            if t.dir[0]==1: return h.intra,t.m
+#        raise
+#    elif dim>1: # 2D or 3D
+#      intra = np.zeros(h.intra.shape) # zero amtrix
+#      inter = np.zeros(h.intra.shape) # zero amtrix
+#      intra = h.intra # initialize
+#      for t in h.hopping: # loop over hoppings
+#        tk = t.m * h.geometry.bloch_phase(t.dir,k) # k hopping
+#        if t.dir[dim-1]==0: intra = intra + tk # add contribution 
+#        if t.dir[dim-1]==1: inter = inter + tk # add contribution 
+#      return intra,inter 
+#    else: raise
+#
+#
 
 
 def get_hopping_dict(h):
     """Return the hopping dictionary"""
     h = h.get_multicell()
     out = dict()
-    out[(0,0,0)] = h.intra
+    out[(0,0,0)] = h.intra.copy()
     for t in h.hopping: 
         if np.max(np.abs(t.m))>1e-7:
-          out[tuple(t.dir)] = t.m # store
+          out[tuple(t.dir)] = t.m.copy() # store
     return out # return
 
 
