@@ -23,12 +23,35 @@ minimum_hopping = 1e-3
 
 
 #except:
+
+def find_close_neighbors(r0,rs,d=2.0):
+    """Return the indexes of the neighbors that are closer than a
+    certain distance"""
+    return find_close_neighbors_jit(np.array(r0),np.array(rs),d=d)
+
+
+@jit(nopython=True)
+def find_close_neighbors_jit(r0,rs,d=2.0):
+    """Return the indexes of the neighbors that are closer than a
+    certain distance"""
+    nout = 0
+    out = np.zeros(len(rs),dtype=np.int_)
+    d2 = d*d
+    dx = rs[:,0] - r0[0]
+    dy = rs[:,1] - r0[1]
+    dz = rs[:,2] - r0[2]
+    dr2 = dx*dx + dy*dy + dz*dz
+    inds = np.arange(0,len(rs))
+    return inds[dr2<d2]
+
+
+
 def find_first_neighbor(r1,r2):
      """Calls the fortran routine"""
      r1 = np.array(r1)
      r2 = np.array(r2)
      nn = number_neighbors_jit(r1.real,r2.real) # number of first neighbors
-     out = np.zeros((nn,2),dtype=np.int) # generate indexes
+     out = np.zeros((nn,2),dtype=np.int_) # generate indexes
      out = find_first_neighbor_jit(r1.real,r2.real,out) # generate all the pairs
      return out
 
@@ -59,7 +82,7 @@ def find_first_neighbor_jit(r1,r2,pairs):
              pairs[out,0] = i
              pairs[out,1] = j
              out += 1 # increase
-    return pairs # number of neighbors
+    return pairs # indexes of the neighbors
 
 
 
@@ -81,23 +104,23 @@ def connections(r1,r2,dr=1.0):
 def parametric_hopping(r1,r2,fc,is_sparse=False):
   """ Generates a parametric hopping based on a function"""
   if is_sparse: # sparse matrix
-    print("Sparse parametric hopping")
-    m = np.matrix([[0.0j for i in range(len(r2))] for j in range(len(r1))])
+    # This should be made more efficient
+#    print("Sparse parametric hopping")
     rows,cols,data = [],[],[]
     for i in range(len(r1)):
       for j in range(len(r2)):
         val = fc(r1[i],r2[j]) # add hopping based on function
         if abs(val) > minimum_hopping: # retain this hopping
-         data.append(val)
-         rows.append(i)
-         cols.append(j)
+            data.append(val)
+            rows.append(i)
+            cols.append(j)
     n = len(r2)
-    m = csc_matrix((data,(rows,cols)),shape=(n,n))
+    m = csc_matrix((data,(rows,cols)),shape=(n,n),dtype=np.complex128)
   #  if not is_sparse: m = m.todense() # dense matrix
     return m
   else:
     n = len(r2)
-    m = np.matrix(np.zeros((n,n),dtype=np.complex)) # complex matrix
+    m = np.array(np.zeros((n,n),dtype=np.complex128)) # complex matrix
     for i in range(len(r1)):
       for j in range(len(r2)):
         m[i,j] = fc(r1[i],r2[j])
@@ -122,8 +145,17 @@ def parametric_hopping_spinful(r1,r2,fc,is_sparse=False):
   return m
 
 
-
-
+# this is a potential speed-up
+#
+#def generate_parametric_hopping(h,f=None,mgenerator=None,
+#             spinful_generator=False):
+#    """Generate a parametric hopping"""
+#    if f is not None and not spinful_generator:
+#        from .specialhopping import entry2matrix
+#        mgen = entry2matrix(f) # create an mgenerator
+#    return generate_parametric_hopping_old(h,f=None,mgenerator=mgen,
+#             spinful_generator=spinful_generator)
+#
 
 
 
@@ -170,7 +202,7 @@ def generate_parametric_hopping(h,f=None,mgenerator=None,
     h.turn_sparse() # turn the matrix sparse
   if not is_sparse and type(h.intra)!=type(np.matrix([[]])):
     h.is_sparse = True
-    h.turn_dense() # turn the matrix sparse
+    h = h.get_dense() # turn the matrix sparse
   if has_spin: # Hamiltonian should be spinful
     h.has_spin = False
     h.turn_spinful()
@@ -190,7 +222,7 @@ def neighbor_distances(g,n=4):
     out = neighbor_distances_jit(r,out) # distances
     out = np.round(out,6) # unique distances
     out = np.unique(out) # unique distances
-    return np.array([out[i+1] for i in range(len(out)-1)]) # return
+    return np.array([out[i+1] for i in range(len(out)-1)])[0:n] # return
 
 
 @jit(nopython=True)
