@@ -5,15 +5,24 @@ def get_operator(self,name,**kwargs):
       """Return the conventional operator"""
       from . import operators
       from . import potentials
+      from .hamiltonians import Hamiltonian
       if type(name) is operators.Operator: return name # return operator
+      if type(name) is Hamiltonian: # if input is an operator 
+          return operators.Operator(name) # return operator
       if type(name) is potentials.Potential or callable(name): 
           out = self.copy()*0. # initialize
           out.add_onsite(name) # add onsite
-          return operators.Operator(out.intra) # return operator
+          return operators.Operator(out.intra,linear=True) # return operator
       if name is None: return None # return operator
       if name=="None": return None
       elif name in ["berry","Berry"]: 
           return operators.get_berry(self,**kwargs)
+      elif name=="site": 
+          return operators.get_site(self,**kwargs)
+      elif name=="angularmomenta": 
+          return operators.get_angular_momenta(self,**kwargs)
+      elif name=="correlator": 
+          return operators.get_correlator_ij(self,**kwargs)
       elif name=="valleyberry": 
           return operators.get_operator_berry(self,"valley",**kwargs)
       elif name=="szvalleyberry": 
