@@ -28,13 +28,15 @@ class Operator():
         if algebra.ismatrix(m):
             self.m = lambda v,k=None: m@v # create dummy function
             self.matrix = m
+            self.linear = True
         elif type(m)==Operator: 
             self.m = m.m
             self.linear = m.linear
+            self.matrix = m.matrix
         elif isinstance(m, numbers.Number): 
             self.m = lambda v,k=None: m*v
         elif callable(m): 
-            self.m = m # as function
+            self.m = m # as function (assume k is a keyword)
         elif type(m)==Hamiltonian: # Hamiltonian type 
             hkgen = m.get_hk_gen()
             self.m = lambda v,k=None: hkgen(k)@v
@@ -47,20 +49,24 @@ class Operator():
         if type(a)==Operator:
             out = Operator(self)
             if self.matrix is not None and a.matrix is not None:
-                out.matrix = self.matrix@a.matrix
+                out.matrix = self.get_matrix()@a.get_matrix()
                 out.m = lambda v,k=None: out.matrix@v # create dummy function
             else: out.m = lambda v,k=None: self.m(a.m(v,k=k),k=k)
             out.linear = self.linear and a.linear
             return out
         elif algebra.ismatrix(a): # matrix type
             if self.matrix is not None: # return a matrix
-                return self.matrix@a # multiply matrices
+                return self.get_matrix()@a # multiply matrices
             else:
                 return self*Operator(a) # convert to operator
         elif algebra.isvector(a): # array type
             return self(a)
         else:
             return self*Operator(a) # convert to operator
+    def trace(self):
+        if self.matrix is not None: 
+            return algebra.trace(self.matrix)
+        else: raise
     def __rmul__(self,a):
         return Operator(a)*self
     def __truediv__(self,a):
@@ -91,7 +97,12 @@ class Operator():
     def __matmul__(self,a): return self*a
     def get_matrix(self,k=None):
         """Return matrix if possible"""
-        if self.matrix is not None: return self.matrix
+        if self.matrix is not None: 
+            if algebra.ismatrix(self.matrix): 
+                return self.matrix
+            else: 
+                print("Operator.matrix has a wrong type",type(self.matrix))
+                raise
     def inv(self):
         """Return the inverse operator"""
         if self.matrix is not None and self.linear: # input is a matrix
@@ -118,9 +129,9 @@ def index(h,n=[0]):
   """Return a projector onto a site"""
   num = len(h.geometry.r)
   val = [1. for i in n]
-  m = csc((val,(n,n)),shape=(num,num),dtype=np.complex)
+  m = csc((val,(n,n)),shape=(num,num),dtype=np.complex128)
   m = h.spinless2full(m) # return matrix with e-h
-  return m@m
+  return Operator(m@m)
 
 
 
@@ -129,7 +140,7 @@ def rfunction2operator(h,f):
     n = len(h.geometry.r)
     val = [f(ri) for ri in h.geometry.r]
     inds = range(n)
-    m = csc((val,(inds,inds)),shape=(n,n),dtype=np.complex)
+    m = csc((val,(inds,inds)),shape=(n,n),dtype=np.complex128)
     return h.spinless2full(m) # return matrix
 
 
@@ -138,7 +149,7 @@ def density2operator(h,d):
     n = len(h.geometry.r)
     if len(d)!=n: raise
     inds = range(n)
-    m = csc((d,(inds,inds)),shape=(n,n),dtype=np.complex)
+    m = csc((d,(inds,inds)),shape=(n,n),dtype=np.complex128)
     return h.spinless2full(m) # return matrix
 
 
@@ -165,7 +176,7 @@ def interface1d(h,cut = 3.):
     if np.abs(y)<cut: data.append(1.) # if it belongs to the interface
     else:  data.append(0.)  # otherwise
   row, col = range(n),range(n)
-  m = csc((data,(row,col)),shape=(n,n),dtype=np.complex)
+  m = csc((data,(row,col)),shape=(n,n),dtype=np.complex128)
   return m # return the operator
 
 
@@ -175,7 +186,7 @@ def get_interface(h,fun=None):
   dind = 1 # index to which divide the positions
   if h.has_spin:  dind *= 2 # duplicate for spin
   if h.has_eh:  dind *= 2  # duplicate for eh
-  iden = csc(np.matrix(np.identity(dind,dtype=np.complex))) # identity matrix
+  iden = csc(np.matrix(np.identity(dind,dtype=np.complex128))) # identity matrix
   r = h.geometry.r # positions
   out = [[None for ri in r] for rj in r] # initialize
   if fun is None: # no input function
@@ -263,7 +274,7 @@ def get_bulk(h,fac=0.8):
     else: return NotImplemented
     from scipy.sparse import diags
     n = len(r) # number of sites
-    out = diags([out],offsets=[0],shape=(n,n),dtype=np.complex) # create matrix
+    out = diags([out],offsets=[0],shape=(n,n),dtype=np.complex128) # create matrix
     m = h.spinless2full(out) # return this matrix
     return m@m # return the square
 
@@ -298,7 +309,7 @@ def get_position(h,mode="z"):
     z = pos[i//dind]
     data.append(z)
   row, col = range(n),range(n)
-  m = csc((data,(row,col)),shape=(n,n),dtype=np.complex)
+  m = csc((data,(row,col)),shape=(n,n),dtype=np.complex128)
   return m # return the operator
 
 
@@ -318,7 +329,7 @@ from .operatortk.spin import get_sz
 def get_z(h):
   """Operator for the calculation of z expectation value"""
   if h.intra.shape[0]==len(h.geometry.z): # if as many positions as entries
-    op = np.zeros(h.intra.shape,dtype=np.complex) # initialize matrix
+    op = np.zeros(h.intra.shape,dtype=np.complex128) # initialize matrix
     for i in range(len(h.geometry.z)):
       op[i,i] = h.geometry.z[i]
     return op
@@ -326,7 +337,7 @@ def get_z(h):
   if h.has_eh: raise
   if not h.has_spin: raise
   if h.has_spin:
-    op = np.zeros(h.intra.shape,dtype=np.complex) # initialize matrix
+    op = np.zeros(h.intra.shape,dtype=np.complex128) # initialize matrix
     for i in range(len(op)//2):   
       op[2*i,2*i+1] = -1j
       op[2*i+1,2*i] = 1j
@@ -345,7 +356,7 @@ def get_rop(h,fun):
   n = h.intra.shape[0]
   row = range(n)
   col = range(n)
-  m = csc((data,(row,col)),shape=(n,n),dtype=np.complex)
+  m = csc((data,(row,col)),shape=(n,n),dtype=np.complex128)
   return m
 
 
@@ -367,7 +378,7 @@ def get_sublattice(h,mode="both"):
   n = h.intra.shape[0]
   row = range(n)
   col = range(n)
-  m = csc((data,(row,col)),shape=(n,n),dtype=np.complex)
+  m = csc((data,(row,col)),shape=(n,n),dtype=np.complex128)
   return m
 
 
@@ -459,7 +470,7 @@ def get_envelop(h,sites=[],d=0.3):
       c = c*d # renormalize all the hoppings
       c[s] = 1.0 # set same atom to 1
       c = c/np.sum(c) # normalize the whole vector
-      c = diags([c],[0],dtype=np.complex) # create matrix
+      c = diags([c],[0],dtype=np.complex128) # create matrix
       out.append(c) # store matrix
     return out # return matrices
 
@@ -591,7 +602,7 @@ def get_layer(self,n=0):
    fac = bool_layer_array(self.geometry,n=n)
    inds = range(len(fac)) # sites
    d = len(fac)
-   m = csc_matrix((fac,(inds,inds)),shape=(d,d),dtype=np.complex) # matrix
+   m = csc_matrix((fac,(inds,inds)),shape=(d,d),dtype=np.complex128) # matrix
    return self.spinless2full(m)
 
 
@@ -621,10 +632,28 @@ def get_potential(self,**kwargs):
 
 
 def get_location(self,r=[0.,0.,0.]):
+    "Return the closest index to a specific location"
     ir = self.geometry.closest_index(r)
     return index(self,n=[ir])
 
 
+def get_site(H,index=0):
+    """Return a projector operator in site number index"""
+    from . import potentials
+    Hp = H*0. # make a dummy copy of the Hamiltonian
+    f = potentials.impurity(H.geometry.r[index],v=1.0) # create a local pot
+    Hp.add_onsite(f) # add onsite energy
+    return Operator(Hp.intra) # return an operator object
 
 
+def get_correlator_ij(H,i=0,j=0):
+    """Return a projector for the correlator ij"""
+    d = len(H.geometry.r)
+    m = np.zeros((d,d),dtype=np.complex128)
+    m[i,j] = 1.0 # set this to nonzero
+    # WARNING, this uses dense matrices
+    m = H.spinless2full(m) # as full matrix
+    return Operator(m) # return operator
 
+
+from .operatortk.angularmomenta import get_angular_momenta
