@@ -38,18 +38,20 @@ class Potential():
 
 
 
-def cnpot(n=4,k=None,v=1.0,angle=0.):
+def cnpot(n=4,k=None,v=1.0,phi=0.,r0=np.array([0.,0.,0.])):
   """Returns a function that generates a potential
   with C_n symmetry"""
   if k is None: raise
   if n==0: return lambda r: v
   if n%2==0: f = np.cos # even 
-  if n%2==1: f = np.sin # even 
+  if n%2==1: f = np.sin # odd 
   def fun(r):
     """Function with the potential"""
+    r = r - r0 # shift
     x0,y0 = r[0],r[1]
     def Rk(k0,angle):
       x0,y0 = k0[0],k0[1] # components
+      angle = angle + phi
       x = np.cos(angle)*x0 + np.sin(angle)*y0
       y = np.cos(angle)*y0 - np.sin(angle)*x0
       return np.array([x,y,k0[2]]) # return rotated vector
@@ -63,7 +65,7 @@ def cnpot(n=4,k=None,v=1.0,angle=0.):
 
 
 
-def aahf1d(n0=0,beta=0.0000001,k=None,b=None,v=1.0,normalize=False):
+def aahf1d(n0=0,beta=1e-7,k=None,b=None,v=1.0,normalize=False):
   """Return the generalized AAHF potential"""
   tau = (1.+np.sqrt(5))/2.
   if b is None: b = 1/tau # default field
@@ -75,7 +77,7 @@ def aahf1d(n0=0,beta=0.0000001,k=None,b=None,v=1.0,normalize=False):
     ys = np.tanh(beta*(np.cos(2.*np.pi*b*ns+k)-np.cos(np.pi*b)))
     ys /= np.tanh(beta)
     return v*ys
-  return fun # return function
+  return Potential(fun) # return function
 
 
 
@@ -266,4 +268,99 @@ def stacking_potential(g,**kwargs):
 
 
 
+def commensurate_vortex_harmonic(g):
+    """Return a skyrmion potential
+    commensurate with the lattice.
+    It may work only for triangular and square lattice
+    """
+    from . import geometry
+    if g.dimensionality==2:
+      a12 = g.a2.dot(g.a1)/(np.sqrt(g.a1.dot(g.a1))*np.sqrt(g.a1.dot(g.a1)))
+      if 0.49<abs(a12)<0.51: 
+          g0 = geometry.triangular_lattice() # triangular
+          # this is a quick fix
+      elif abs(a12)<0.01: 
+          g0 = geometry.square_lattice() # square
+      else: raise # not implemented
+    else: raise
+    h0 = g0.get_hamiltonian()
+    h0.add_rashba(1.0) # add Rashba SOC
+    r2f = g.get_k2K_generator() # get function
+    hk = h0.get_hk_gen() # generator
+    da = np.sqrt(g.a1.dot(g.a1))/2 # length of the vector
+    def fun(r):
+        rf = r/da
+        rf = r2f(rf) # convert to fractional
+#        rf = Ri@rf
+        m = hk(rf) # evaluate Hamiltonian
+        mx,my = m[0,1].real,-m[0,1].imag
+        return np.array([mx,my,0.])
+    return Potential(fun)
+
+
+def commensurate_skyrmion_harmonic(g):
+    """Create a skyrmion commensurate with the lattice"""
+    # this function as nood as good as it could
+    # probably the vectors should have the same length
+    fv = commensurate_vortex(g) # get the vortex part
+    vs = [fv(ri) for ri in g.r] # magnetizations
+    vmax = np.max([vi.dot(vi) for vi in vs]) # absolute value
+    fz = commensurate_potential(g,minmax=[-1.,1.],n=6,k=1) # z component
+    def fun(r):
+        m = fv(r) # xy component
+        dz = vmax - m.dot(m) # value of z component
+        return m + np.array([0.,0.,fz(r)])*dz
+    return Potential(fun)
+
+
+
+def commensurate_skyrmion(g,r0=None):
+    """Return a conmmensurate skyrmion"""
+    if r0 is not None: rc = r0
+    else: rc = np.mean(g.r,axis=0) # average position
+    da = np.sqrt(g.a1.dot(g.a1)) # average radius
+    R = da/4. # radius of the skyrmion
+    def fun(r):
+        r = r - rc # radious with respect to the center
+        r = r/R # normalize to the radious of the skyrmion
+        dr = np.sqrt(r.dot(r)) # distance to center
+        if dr<1.: 
+            mz = np.sqrt(1. - dr**2) # z component
+            return r + np.array([0.,0.,mz])
+        if 1.<dr<1.5: 
+            mxy = r/dr*(1.5-dr)
+            mz = np.sqrt(1. - mxy.dot(mxy)) # z component
+            return mxy + np.array([0.,0.,-mz])
+        else:
+            return np.array([0.,0.,-1.])
+    return Potential(fun)
+
+
+
+def circle(r0,v=0.,R=1.1,g=None,**kwargs):
+    """Create the potential for a circle"""
+    def f(r):
+        dr = r-r0
+        if dr.dot(dr)<R: return v
+        else: return 0.
+    out = Potential(f) # return the potential
+    if g is None: return out
+    else: return periodic_replicate(out,g,**kwargs)
+
+
+
+def periodic_replicate(P,g,n=3):
+    """Enforce a potential to have the periodicity
+    in space of the lattice"""
+    dl = g.neighbor_directions(n) # list with neighboring cells to take
+    def fout(r):
+        o = 0.
+        for d in dl:
+            ri = r + g.a1*d[0] + g.a2*d[1] + g.a3*d[2]
+            o = o + P(ri) # add contribution
+        return o
+    return Potential(fout)
+
+
+
 
