@@ -2,12 +2,6 @@ import numpy as np
 from scipy.sparse import csc_matrix
 from numba import jit
 
-#try:
-#    from . import specialhoppingf90
-##    raise
-#    use_fortran=True
-#except:
-#    use_fortran=False
 use_fortran=False
 
 
@@ -48,6 +42,11 @@ def twisted(cutoff=5.0,ti=0.3,lambi=8.0,t=1.0,
     return fun
 
 
+def real_graphene():
+    return twisted(ti=0.15,lamb=0.5)
+
+
+
 def twisted_matrix(cutoff=5.0,ti=0.3,lambi=8.0,mint=1e-5,
         t=1.0,lamb=12.0,dl=3.0,lambz=10.0,**kwargs):
   """Function capable of returning the hopping matrix
@@ -64,7 +63,7 @@ def twisted_matrix(cutoff=5.0,ti=0.3,lambi=8.0,mint=1e-5,
 #      ts = ts[0:nout]
 #      ii = ii[0:nout]
 #      jj = jj[0:nout]
-#      out = csc_matrix((ts,(ii-1,jj-1)),shape=(nr,nr),dtype=np.complex) # matrix
+#      out = csc_matrix((ts,(ii-1,jj-1)),shape=(nr,nr),dtype=np.complex128) # matrix
 #      return out
 #    return funhop # return function
   if True:
@@ -91,8 +90,25 @@ def multilayer(ti=0.3,dz=3.0):
     return fhop
 
 def entry2matrix(f):
-    def fout(rs1,rs2):
-        return np.array([[f(r1,r2) for r1 in rs1] for r2 in rs2]).T
+    """Transform a function that returns a hopping into a function
+    that returns the hopping matrix"""
+    try:
+        fnum = jit(f,nopython=True) # jit function
+        def fout(rs1,rs2):
+            n1 = len(rs1)
+            n2 = len(rs2)
+            out = np.zeros((n1,n2),dtype=np.complex128)
+            for i in range(n1):
+                r1 = rs1[i]
+                for j in range(n2):
+                    r2 = rs2[j]
+                    out[i,j] = fnum(r1,r2)
+            return out
+        return jit(fout,nopython=True)
+    except:
+        print("Warning, hopping cannot be jitd")
+        def fout(rs1,rs2):
+            return np.array([[f(r1,r2) for r1 in rs1] for r2 in rs2]).T
     return fout
 
 
@@ -144,7 +160,7 @@ def distance_hopping_matrix(vs,ds):
         r1 = np.array(r1)
         r2 = np.array(r2)
         n = len(r1)
-        out = np.zeros((n,n),dtype=np.complex) # output
+        out = np.zeros((n,n),dtype=np.complex128) # output
         return distance_hopping_matrix_jit(r1,r2,vs,ds*ds,out) 
     return mgenerator
 
@@ -228,7 +244,7 @@ def twisted_matrix_python(cutoff=10,**kwargs):
   def tij(rs1,rs2): # function to return
       nr = len(rs1) # length
       nmax = nr*int(10*cutoff**2) # maximum number of hoppings
-      data = np.zeros(nmax,dtype=np.complex) # data
+      data = np.zeros(nmax,dtype=np.complex128) # data
       ii = np.zeros(nmax,dtype=int) # index
       jj = np.zeros(nmax,dtype=int) # index
       ii,jj,data,nk = twisted_matrix_jit(np.array(rs1),np.array(rs2),
@@ -237,7 +253,7 @@ def twisted_matrix_python(cutoff=10,**kwargs):
       ii = ii[0:nk] # only nonzero
       jj = jj[0:nk] # only nonzero
       data = data[0:nk] # only nonzero
-      out = csc_matrix((data,(ii,jj)),shape=(nr,nr),dtype=np.complex) # matrix
+      out = csc_matrix((data,(ii,jj)),shape=(nr,nr),dtype=np.complex128) # matrix
       out.eliminate_zeros()
       return out
   return tij
