@@ -18,6 +18,7 @@ arpack_maxiter = 10000
 
 def fermi_surface(h,write=True,output_file="FERMI_MAP.OUT",
                     e=0.0,nk=50,nsuper=1,reciprocal=True,
+                    k0 = np.array([0.,0.]),
                     delta=None,refine_delta=1.0,operator=None,
                     mode='eigen',num_waves=2,info=False):
     """Calculates the Fermi surface of a 2d system"""
@@ -32,7 +33,7 @@ def fermi_surface(h,write=True,output_file="FERMI_MAP.OUT",
     hk_gen = h.get_hk_gen() # gets the function to generate h(k)
     kxs = np.linspace(-nsuper,nsuper,nk)  # generate kx
     kys = np.linspace(-nsuper,nsuper,nk)  # generate ky
-    iden = np.identity(h.intra.shape[0],dtype=np.complex)
+    iden = np.identity(h.intra.shape[0],dtype=np.complex128)
     kxout = []
     kyout = []
     if reciprocal: R = h.geometry.get_k2K_generator() # get function
@@ -72,8 +73,9 @@ def fermi_surface(h,write=True,output_file="FERMI_MAP.OUT",
     for x in kxs:
       for y in kxs:
         rs.append([x,y,0.]) # store
+    k0 = np.array([k0[0],k0[1],0.])
     def getf(r): # function to compute FS
-        k = R(r)
+        k = R(r) + k0
         hk = hk_gen(k) # get hamiltonian
         return get_weight(hk,k=k)
     rs = np.array(rs) # transform into array
@@ -302,7 +304,7 @@ def real_space_vev(h,operator=None,nk=1,nrep=3,name="REAL_SPACE_VEV.OUT",
 def total_energy(h,nk=10,nbands=None,use_kpm=False,random=False,
         kp=None,mode="mesh",tol=1e-1,fermi=0.0):
   """Return the total energy"""
-  if nbands is None: h.turn_dense()
+  if nbands is None: h = h.get_dense()
   if h.is_sparse and not use_kpm: 
       if nbands is None:
         print("Sparse Hamiltonian but no bands given, taking 20")
@@ -343,29 +345,7 @@ def total_energy(h,nk=10,nbands=None,use_kpm=False,random=False,
 
 
 
-
-
-
-def eigenvalues(h0,nk=10,notime=True):
-    """Return all the eigenvalues of a Hamiltonian"""
-    from . import klist
-    h = h0.copy() # copy hamiltonian
-    h.turn_dense()
-    ks = klist.kmesh(h.dimensionality,nk=nk) # get grid
-    hkgen = h.get_hk_gen() # get generator
-    if parallel.cores==1:
-      es = [] # empty list
-      if not notime: est = timing.Testimator(maxite=len(ks))
-      for k in ks: # loop
-          if not notime: est.iterate()
-          es += algebra.eigvalsh(hkgen(k)).tolist() # add
-    else:
-        f = lambda k: algebra.eigvalsh(hkgen(k)) # add
-        es = parallel.pcall(f,ks) # call in parallel
-        es = np.array(es)
-        es = es.reshape(es.shape[0]*es.shape[1])
-    return es # return all the eigenvalues
-
+from .filling import eigenvalues
 
 
 
@@ -440,55 +420,14 @@ def pairing_map(h,**kwargs):
 
 
 
+from .filling import set_filling
+from .filling import get_fermi_energy
 
-def set_filling(h,filling=0.5,nk=10,extrae=0.,
-    mode="ED",**kwargs):
-    """
-    Set the filling of a Hamiltonian
-    - nk = 10, number of kpoints in each direction
-    - filling = 0.5, filling of the lattice
-    - extrae = 0.0, number of extra electrons
-    """
-    if h.has_eh: # quick workaround
-        ef = h.get_fermi4filling(filling,nk=nk) # fermi energy
-        h.add_onsite(-ef)
-        return
-    fill = filling + extrae/h.intra.shape[0] # filling
-    n = h.intra.shape[0]
-    use_kpm = False
-    if n>algebra.maxsize: # use the KPM method
-        mode="KPM"
-        print("Using KPM in set_filling")
-    if mode=="KPM": # use KPM
-        es,ds = h.get_dos(energies=np.linspace(-5.0,5.0,1000),
-                mode="KPM",nk=nk,**kwargs)
-        from scipy.integrate import cumtrapz
-        di = cumtrapz(ds,es)
-        ei = (es[0:len(es)-1] + es[1:len(es)])/2.
-        di /= di[len(di)-1] # normalize
-        from scipy.interpolate import interp1d
-        f = interp1d(di,ei) # interpolating function
-        efermi = f(fill) # get the fermi energy
-    elif mode=="ED": # dense Hamiltonian, use ED
-        es = eigenvalues(h,nk=nk,notime=True)
-        efermi = get_fermi_energy(es,fill)
-    else: raise
-    h.shift_fermi(-efermi) # shift the fermi energy
-
-
-def get_fermi_energy(es,filling,fermi_shift=0.0):
-  """Return the Fermi energy"""
-  ne = len(es) ; ifermi = int(round(ne*filling)) # index for fermi
-  if ifermi>=ne: ifermi = ne-1
-  sorte = np.sort(es) # sorted eigenvalues
-  if ifermi==0: return sorte[0] + fermi_shift
-  fermi = (sorte[ifermi-1] + sorte[ifermi])/2.+fermi_shift # fermi energy
-  return fermi
 
 
 def get_fermi4filling(h,filling,nk=8):
     """Return the fermi energy for a certain filling"""
-    if h.has_eh: 
+    if h.has_eh: # this is an approximation, accurate version to be written 
         h0 = h.copy()
         h0.remove_nambu()
         return get_fermi4filling(h0,filling,nk=nk) # workaround
@@ -545,7 +484,7 @@ def lowest_energies(h,n=4,k=None,**kwargs):
 def get_bandwidth(self,**kwargs):
     """Return the bandwidth of the Hamiltonian"""
     from .gap import optimize_energy 
-    self.turn_dense() # dense matrix
+    self = self.get_dense() # dense matrix
     emin = optimize_energy(self,mode="bottom",**kwargs)
     emax = optimize_energy(self,mode="top",**kwargs)
     return (emin,emax)
