@@ -127,8 +127,6 @@ def replicate3d_jit(rs,a1,a2,a3,n1,n2,n3,ro):
   return ro # return positions
 
 
-#@jit(nopython=True)
-#@jit
 def return_unique(rs1,rs2):
   """Return only those positions in rs1 that do not appear in rs2"""
   rout = []
@@ -140,10 +138,6 @@ def return_unique(rs1,rs2):
 
 
 
-#def request(g,nat,ntries=20):
-#  """Request a unit cell with as many atoms"""
-
-
 def target_angle_volume(g,angle=None,n=5,volume=None,same_length=False):
     """Return a supercell, targetting a certain new angle between vectors"""
     if g.dimensionality!=2: raise # only for 2d
@@ -283,7 +277,9 @@ def supercell3d(g,n1=1,n2=1,n3=1):
 
 
 
-
+def turn_orthorhombic(g):
+    """Transform a geometry into an orthorombic one"""
+    return target_angle(g,angle=0.5)
 
 
 
