@@ -4,6 +4,7 @@ from scipy.sparse import coo_matrix, bmat, csc_matrix
 import scipy.sparse as sp
 from scipy.sparse import issparse
 from . import algebra
+from .algebra import dagger
 
 
 
@@ -42,7 +43,7 @@ def get_nambu_tauz(m,has_eh=False):
   raise # this function is not consistent with the Nambu notation (see below)
   n = m.shape[0] # number of sites 
   if has_eh: n = n//2 # half
-  mout = np.matrix(np.zeros((n*2,n*2)),dtype=np.complex) # define matrix
+  mout = np.matrix(np.zeros((n*2,n*2)),dtype=np.complex128) # define matrix
   for ii in range(n): # loop over index
     mout[2*ii,2*ii] = 1. # assign
     mout[2*ii+1,2*ii+1] = -1. # assign
@@ -170,14 +171,14 @@ projh = csc_matrix([[0.,0.,0.,0.],[0.,0.,0.,0.],[0.,0.,1.,0.],[0.,0.,0.,1.]])
 deltauu = csc_matrix([[0.,0.,0.,0.],[0.,0.,0.,0.],[0.,0.,0.,0.],[-1.,0.,0.,0.]])
 # and the rest are simple
 deltadd = csc_matrix([[0.,0.,0.,0.],[0.,0.,0.,0.],[0.,1.,0.,0.],[0.,0.,0.,0.]])
-deltauu = deltauu.H
-deltadd = deltadd.H
+deltauu = dagger(deltauu)
+deltadd = dagger(deltadd)
 deltax = (deltadd - deltauu)/2.
 deltay = (deltadd + deltauu)/2j
 # this one is tricky, we only want the antisymmetric part
 deltaz = csc_matrix([[0.,0.,0.,0.],[0.,0.,0.,0.],[1.,0.,0.,0.],[0.,-1.,0.,0.]])/2.
 
-deltaz = deltaz.H
+deltaz = dagger(deltaz)
 
 
 
@@ -236,7 +237,7 @@ def add_swave(delta=0.0,is_sparse=False,rs=None):
       data.append(dr)  # up e dn h
       data.append(dr)  # dne up h
     iis = range(n*2) # indexes
-    coupling = csc_matrix((data,(iis,iis)),dtype=np.complex)  # generate matrix
+    coupling = csc_matrix((data,(iis,iis)),dtype=np.complex128)  # generate matrix
   else:
     coupling = sp.identity(n*2)*delta # delta matrix
   zero = coupling*0.
@@ -311,11 +312,12 @@ def add_pairing(deltas=[[0.,0],[0.,0.]],is_sparse=True,r1=[],r2=[]):
   n = len(r1)  # number of sites
   pout = [[None for i in range(n)] for j in range(n)] # initialize None matrix
   # zeros in the diagonal
-#  for i in range(n): bout[i][i] = csc_matrix(np.zeros((4,4),dtype=np.complex))
+#  for i in range(n): bout[i][i] = csc_matrix(np.zeros((4,4),dtype=np.complex128))
   for i in range(n): # loop over sites
     for j in range(n): # loop over sites
       pout[i][j] = get_pmatrix(r1[i],r2[j]) # get this pairing
-  diag = csc_matrix(np.zeros((2*len(r1),2*len(r1)),dtype=np.complex)) # diag
+  diag = sp.identity(2*len(r1))*0. # zero matrix
+#  diag = csc_matrix(np.zeros((2*len(r1),2*len(r1)),dtype=np.complex128)) # diag
   pout = bmat(pout) # convert to block matrix
 #  mout = [[diag,pout],[np.conjugate(pout),diag]] # output matrix
   mout = [[diag,pout],[None,diag]] # output matrix
@@ -327,44 +329,21 @@ def add_pairing(deltas=[[0.,0],[0.,0.]],is_sparse=True,r1=[],r2=[]):
 
 
 
-
-
-def block2nambu_matrix(m):
-  '''Reorder a matrix that has electrons and holes, so that
-  the order resembles the Nambu spinor in each site.
-  The initial matrix is
-  H D
-  D H
-  The output is a set of block matrices for each site in the 
-  Nambu form'''
-  R = np.matrix(np.zeros(m.shape)) # zero matrix
-  nr = m.shape[0]//4 # number of positions
-  for i in range(nr): # electrons
-    R[2*i,4*i] = 1.0 # up electron
-    R[2*i+1,4*i+1] = 1.0 # down electron
-    R[2*i+2*nr,4*i+2] = 1.0 # down holes
-    R[2*i+1+2*nr,4*i+3] = 1.0 # up holes
-  R = csc_matrix(R) # to sparse
-  return R
-
-
-def block2nambu(m):
-    R = block2nambu_matrix(m)
-    Rh = np.conjugate(R.T)
-    return Rh@m@R
+from .sctk.reorder import block2nambu
+from .sctk.reorder import block2nambu_matrix
+from .sctk.reorder import nambu2block
 
 reorder = block2nambu
 
-def nambu2block(m):
-    R = block2nambu_matrix(m)
-    Rh = np.conjugate(R.T)
-    return R@m@Rh
-
 
 from .sctk.extract import extract_pairing
 from .sctk.extract import extract_singlet_pairing
 from .sctk.extract import extract_triplet_pairing
 from .sctk.pairing import pairing_generator
+from .sctk.fastdeltaud import hopping2deltaud
+
+
+
 
 def add_pairing_to_hamiltonian(self,**kwargs):
     """ Add a general pairing matrix to a Hamiltonian"""
@@ -373,7 +352,7 @@ def add_pairing_to_hamiltonian(self,**kwargs):
     self.turn_nambu() # add electron hole terms
     r = self.geometry.r # positions 
     m = add_pairing(df,r1=r,r2=r) # intra cell
-    self.intra = self.intra + m + m.H
+    self.intra = self.intra + m + algebra.dagger(m)
     if self.dimensionality>0:
       if not self.is_multicell: # for multicell hamiltonians
         self.turn_multicell()
@@ -390,14 +369,14 @@ def add_pairing_to_hamiltonian(self,**kwargs):
         if np.max(np.abs(m))>0.0001: # non zero
             self.hopping.append(Hopping(d=d,m=m)) # add pairing
         if np.max(np.abs(m2))>0.0001: # non zero
-            self.hopping.append(Hopping(d=-np.array(d),m=m2.H)) # add pairing
+            self.hopping.append(Hopping(d=-np.array(d),m=dagger(m2))) # add pairing
       from .multicell import collect_hopping
       self.hopping = collect_hopping(self)
 
 
-iden = np.matrix([[1.,0.],[0.,1.]],dtype=np.complex)
-taux = np.matrix([[0.,1.],[1.,0.]],dtype=np.complex)
-tauz = np.matrix([[1.,0.],[0.,-1.]],dtype=np.complex)
+iden = np.matrix([[1.,0.],[0.,1.]],dtype=np.complex128)
+taux = np.matrix([[0.,1.],[1.,0.]],dtype=np.complex128)
+tauz = np.matrix([[1.,0.],[0.,-1.]],dtype=np.complex128)
 
 
 
@@ -405,7 +384,7 @@ tauz = np.matrix([[1.,0.],[0.,-1.]],dtype=np.complex)
 def extract_euphdn(m):
     """Extract electron up hole down sector"""
     n = m.shape[0]//2 # half the dimension
-    out = np.matrix(np.zeros((n,n),dtype=np.complex))
+    out = np.matrix(np.zeros((n,n),dtype=np.complex128))
     for i in range(n):
       for j in range(n):
         out[i,j] = m[2*i,2*j]
@@ -440,7 +419,7 @@ def get_nambu2signless(m):
         dd.append(1.)
         dd.append(-1.)
     from scipy.sparse import diags
-    return diags([dd],[0],shape=(4*n,4*n),dtype=np.complex) # create matrix
+    return diags([dd],[0],shape=(4*n,4*n),dtype=np.complex128) # create matrix
 
 
 
