@@ -10,15 +10,21 @@ from . import inout
 from . import timing
 from . import algebra
 from . import parallel
+from numba import jit
 
 arpack_tol = algebra.arpack_tol
 arpack_maxiter = algebra.arpack_maxiter
 
 
-def write_berry(h,kpath=None,dk=0.01,window=None,max_waves=None,nk=600,
+
+
+
+def get_berry_curvature_path(h,kpath=None,dk=0.01,
+      window=None,max_waves=None,nk=600,
       mode="Wilson",delta=0.001,reciprocal=False,operator=None,
       silent = True):
-    """Calculate and write in file the Berry curvature"""
+    """Calculate and write in file the Berry curvature
+    in a certain kpath"""
     operator = get_operator(h,operator)
     kpath = klist.get_kpath(h.geometry,kpath=kpath,nk=nk) # take default kpath
     tr = timing.Testimator("BERRY CURVATURE",silent=silent)
@@ -48,8 +54,8 @@ def write_berry(h,kpath=None,dk=0.01,window=None,max_waves=None,nk=600,
     return np.array(range(len(m[0]))),m[2]
 
 
-
-
+# alias for compatibility
+write_berry = get_berry_curvature_path
 
 
 
@@ -119,115 +125,120 @@ from .topologytk.occstates import occupied_states
 from .topologytk.occstates import occ_states2d
 
 
-
-
-def uij(wf1,wf2):
-  """ Calcultes the matrix product of two sets of input wavefunctions"""
-  out =  np.matrix(np.conjugate(wf1))@(np.matrix(wf2).T)  # faster way
-  return out
-
-
-def uij_slow(wf1,wf2):
-  m = np.matrix(np.zeros((len(wf1),len(wf2)),dtype=np.complex))
-  for i in range(len(wf1)):
-    for j in range(len(wf2)):
-      m[i,j] = np.conjugate(wf1[i]).dot(wf2[j])
-  return m
+from .topologytk.overlap import uij
 
 
 def precise_chern(h,dk=0.01, mode="Wilson",delta=0.0001,operator=None):
-  """ Calculates the chern number of a 2d system """
-  from scipy import integrate
-  err = {"epsabs" : 1.0, "epsrel": 1.0,"limit" : 10}
-#  err = [err,err]
-  def f(x,y): # function to integrate
-    if mode=="Wilson":
-      return berry_curvature(h,np.array([x,y]),dk=dk)
-    if mode=="Green":
-       f2 = h.get_gk_gen(delta=delta) # get generator
-       return berry_green(f2,k=[x,y,0.],operator=operator) 
-  c = integrate.dblquad(f,0.,1.,lambda x : 0., lambda x: 1.,epsabs=0.01,
-                          epsrel=0.01)
-  chern = c[0]/(2.*np.pi)
-  open("CHERN.OUT","w").write(str(chern)+"\n")
-  return chern
+    """ Calculates the chern number of a 2d system """
+    from scipy import integrate
+    err = {"epsabs" : 1.0, "epsrel": 1.0,"limit" : 10}
+    def f(x,y): # function to integrate
+      if mode=="Wilson":
+        return berry_curvature(h,np.array([x,y]),dk=dk)
+      elif mode=="Green":
+         f2 = h.get_gk_gen(delta=delta) # get generator
+         return berry_green(f2,k=[x,y,0.],operator=operator) 
+      else: raise
+    c = integrate.dblquad(f,0.,1.,lambda x : 0., lambda x: 1.,epsabs=0.01,
+                            epsrel=0.01)
+    chern = c[0]/(2.*np.pi)
+    open("CHERN.OUT","w").write(str(chern)+"\n")
+    return chern
 
 
 def hall_conductivity(h,dk=-1,n=1000):
-  c = 0.0 
-  nk = int(np.sqrt(n)) # estimate
-  if dk<0: dk = 1./float(2*nk) # automatic dk
-  for i in range(n):
-    k = np.random.random(2) # random kpoint
-    c += berry_curvature(h,k,dk=dk)
-  c = c/(2*np.pi*n) # normalize
-  return c
+    c = 0.0 
+    nk = int(np.sqrt(n)) # estimate
+    if dk<0: dk = 1./float(2*nk) # automatic dk
+    for i in range(n):
+      k = np.random.random(2) # random kpoint
+      c += berry_curvature(h,k,dk=dk)
+    c = c/(2*np.pi*n) # normalize
+    return c
 
 
 
 
 def mesh_chern(h,dk=-1,nk=10,delta=0.0001,mode="Wilson",
         operator=None,kmesh=None):
-  """ Calculates the chern number of a 2d system """
-  c = 0.0
-  ks = [] # array for kpoints
-  bs = [] # array for berrys
-  if dk<0: dk = 1./float(2*nk) # automatic dk
-  if kmesh is not None: # infer the dk of the mesh
-      dk = klist.infer_kmesh_dk(kmesh,d=2)
-  if operator is not None and mode=="Wilson":
-    print("Switching to Green mode in topology")
-    mode="Green"
-  # create the function
-  def fberry(k): # function to integrate
-    if mode=="Wilson":
-      return berry_curvature(h,k,dk=dk)
-    if mode=="Green":
-       f2 = h.get_gk_gen(delta=delta) # get generator
-       return berry_green(f2,k=[k[0],k[1],0.],operator=operator) 
-  ##################
-  if kmesh is None: # no kmesh provided
-      ks = klist.kmesh(h.dimensionality,nk=nk) # get the mesh
-  else: ks = kmesh # use the provided kmesh
-  ik = 0
-  bs = parallel.pcall(fberry,ks) # compute all the Berry curvatures
-  # write in file
-  fo = open("BERRY_CURVATURE.OUT","w") # open file
-  for (k,b) in zip(ks,bs):
-    fo.write(str(k[0])+"   ")
-    fo.write(str(k[1])+"   ")
-    fo.write(str(b)+"\n")
-  fo.close() # close file
-  ################
-  c = np.sum(bs) # sum berry curvatures
-  if kmesh is None: # no kmesh provided
-      c = c/(2.*np.pi*nk*nk) # normalize
-  else: # kmesh is given
-      den = klist.infer_kmesh_density(kmesh,d=2) # infer the volume
-      c = den*c/(2.*np.pi) # normalize
-  open("CHERN.OUT","w").write(str(c)+"\n")
-  return c
+    """ Calculates the chern number of a 2d system """
+    c = 0.0
+    ks = [] # array for kpoints
+    bs = [] # array for berrys
+    if dk<0: dk = 1./float(2*nk) # automatic dk
+    if kmesh is not None: # infer the dk of the mesh
+        dk = klist.infer_kmesh_dk(kmesh,d=2)
+    if operator is not None and mode=="Wilson":
+      print("Switching to Green mode in topology")
+      mode="Green"
+    # create the function
+    def fberry(k): # function to integrate
+      if mode=="Wilson":
+        return berry_curvature(h,k,dk=dk)
+      if mode=="Green":
+         f2 = h.get_gk_gen(delta=delta) # get generator
+         return berry_green(f2,k=[k[0],k[1],0.],operator=operator) 
+    ##################
+    if kmesh is None: # no kmesh provided
+        ks = klist.kmesh(h.dimensionality,nk=nk) # get the mesh
+    else: ks = kmesh # use the provided kmesh
+    ik = 0
+    bs = parallel.pcall(fberry,ks) # compute all the Berry curvatures
+    # write in file
+    fo = open("BERRY_CURVATURE.OUT","w") # open file
+    for (k,b) in zip(ks,bs):
+      fo.write(str(k[0])+"   ")
+      fo.write(str(k[1])+"   ")
+      fo.write(str(b)+"\n")
+    fo.close() # close file
+    ################
+    c = np.sum(bs) # sum berry curvatures
+    if kmesh is None: # no kmesh provided
+        c = c/(2.*np.pi*nk*nk) # normalize
+    else: # kmesh is given
+        den = klist.infer_kmesh_density(kmesh,d=2) # infer the volume
+        c = den*c/(2.*np.pi) # normalize
+    open("CHERN.OUT","w").write(str(c)+"\n")
+    return c
+
+
+def get_berry_curvature(self,**kwargs):
+    """Compute Berry curvature"""
+    if self.non_hermitian: # non Hermitian case
+        from .nonhermitiantk.nhmethods import get_berry_curvature as BNH
+        return BNH(self,**kwargs)
+    else: # Hermitian case
+        return get_berry_curvature_master(self,**kwargs)
 
 
 
-def get_berry_curvature(h,dk=None,nk=100,reciprocal=True,nsuper=1,window=None,
+
+
+def get_berry_curvature_master(h,dk=None,nk=100,
+        reciprocal=True,nsuper=1,window=None,
+        kpath=None,
                max_waves=None,mode="Wilson",delta=0.001,operator=None,
                write=True,verbose=0):
     """ Return the Berry curvature in 2D reciprocal space """
+    # get the right kpoints
+    if kpath is None: # no kpath, just to a grid
+        ks = [] # list with kpoints
+        for x in np.linspace(-nsuper,nsuper,nk,endpoint=False):
+          for y in np.linspace(-nsuper,nsuper,nk,endpoint=False):
+              ks.append([x,y,0.])
+    else: # kpath provided
+        ks = h.geometry.get_kpath(kpath,nk=nk)
+        reciprocal = False # if given, assume they are in standard way
+    ks = np.array(ks) # convert to array
+    ##############
     if operator is not None: mode="Green" # Green function mode
     c = 0.0
-    ks = [] # array for kpoints
     if dk is None: dk = 1./float(2*nk) # automatic dk
     if reciprocal: R = np.array(h.geometry.get_k2K())
     else: R = np.array(np.identity(3))
     nt = nk*nk # total number of points
     ik = 0
-    ks = [] # list with kpoints
     from . import parallel
-    for x in np.linspace(-nsuper,nsuper,nk,endpoint=False):
-      for y in np.linspace(-nsuper,nsuper,nk,endpoint=False):
-          ks.append([x,y,0.])
-    ks = np.array(ks) # convert to array
     if verbose>0: tr = timing.Testimator("BERRY CURVATURE",maxite=len(ks))
     def fp(ki): # function to compute the Berry curvature
         if parallel.cores == 1: 
@@ -254,56 +265,50 @@ def get_berry_curvature(h,dk=None,nk=100,reciprocal=True,nsuper=1,window=None,
 
 berry_map = get_berry_curvature # alias
 
-
-def smooth_gauge(w1,w2):
-  """Perform a gauge rotation so that the second set of waves are smooth
-  with respect to the first one"""
-  m = uij(w1,w2) # matrix of wavefunctions
-  U, s, V = np.linalg.svd(m, full_matrices=True) # sing val decomp
-  R = (U@V).H # rotation matrix
-  wnew = w2.copy()*0j # initialize
-  wold = w2.copy() # old waves
-  for ii in range(R.shape[0]):
-    for jj in range(R.shape[0]):
-      wnew[ii] += R[jj,ii]*wold[jj]
-  return wnew
+from .topologytk.wannier import smooth_gauge
 
 
+def z2_wannier_centers(h,full=False,**kwargs):
+    """Return the Wannier centers for the Z2 invariant"""
+    return wannier_centers(h,full=False,**kwargs)
 
 
-def z2_vanderbilt(h,nk=30,nt=100,nocc=None,full=False):
-  """ Calculate Z2 invariant according to Vanderbilt algorithm"""
-  out = [] # output list
-  path = np.linspace(0.,1.,nk) # set of kpoints
-  fo = open("WANNIER_CENTERS.OUT","w")
-  if full:  ts = np.linspace(0.,1.0,nt,endpoint=False)
-  else:  ts = np.linspace(0.,0.5,nt,endpoint=False)
-  wfall = [[occ_states2d(h,np.array([k,t,0.,])) for k in path] for t in ts] 
-  # select a continuos gauge for the first wave
-  for it in range(len(ts)-1): # loop over ts
-    wfall[it+1][0] = smooth_gauge(wfall[it][0],wfall[it+1][0]) 
-  for it in range(len(ts)): # loop over t points
-    row = [] # empty list for this row
-    t = ts[it] # select the t point
-    wfs = wfall[it] # get set of waves 
-    for i in range(len(wfs)-1):
-      wfs[i+1] = smooth_gauge(wfs[i],wfs[i+1]) # transform into a smooth gauge
-#      m = uij(wfs[i],wfs[i+1]) # matrix of wavefunctions
-    m = uij(wfs[0],wfs[len(wfs)-1]) # matrix of wavefunctions
-    evals = lg.eigvals(m) # eigenvalues of the rotation 
-    x = np.angle(evals) # phase of the eigenvalues
-    fo.write(str(t)+"    ") # write pumping variable
-    row.append(t) # store
-    for ix in x: # loop over phases
-      fo.write(str(ix)+"  ")
-      row.append(ix) # store
-    fo.write("\n")
-    out.append(row) # store
-  fo.close()
-  return np.array(out).transpose() # transpose the map
+z2_vanderbilt = z2_wannier_centers # for compatibility
 
 
-def z2_invariant(h,nk=20,nt=20,nocc=None):
+def wannier_centers(h,nk=30,nt=100,nocc=None,full=False):
+    """ Calculate Z2 invariant according to Vanderbilt algorithm"""
+    out = [] # output list
+    path = np.linspace(0.,1.,nk) # set of kpoints
+    fo = open("WANNIER_CENTERS.OUT","w")
+    if full:  ts = np.linspace(0.,1.0,nt,endpoint=False)
+    else:  ts = np.linspace(0.,0.5,nt,endpoint=False)
+    wfall = [[occ_states2d(h,np.array([k,t,0.,])) for k in path] for t in ts] 
+    # select a continuos gauge for the first wave
+    for it in range(len(ts)-1): # loop over ts
+      wfall[it+1][0] = smooth_gauge(wfall[it][0],wfall[it+1][0]) 
+    for it in range(len(ts)): # loop over t points
+      row = [] # empty list for this row
+      t = ts[it] # select the t point
+      wfs = wfall[it] # get set of waves 
+      for i in range(len(wfs)-1):
+        wfs[i+1] = smooth_gauge(wfs[i],wfs[i+1]) # transform into a smooth gauge
+  #      m = uij(wfs[i],wfs[i+1]) # matrix of wavefunctions
+      m = uij(wfs[0],wfs[len(wfs)-1]) # matrix of wavefunctions
+      evals = lg.eigvals(m) # eigenvalues of the rotation 
+      x = np.angle(evals) # phase of the eigenvalues
+      fo.write(str(t)+"    ") # write pumping variable
+      row.append(t) # store
+      for ix in x: # loop over phases
+        fo.write(str(ix)+"  ")
+        row.append(ix) # store
+      fo.write("\n")
+      out.append(row) # store
+    fo.close()
+    return np.array(out).transpose() # transpose the map
+
+
+def z2_invariant(h,nk=60,nt=60,nocc=None):
   """Compute Z2 invariant with pumping of Wannier centers"""
   return wannier_winding(h,nk=nk,nt=nt,nocc=nocc,full=False) 
 
@@ -311,112 +316,89 @@ def z2_invariant(h,nk=20,nt=20,nocc=None):
 
 
 def chern(h,**kwargs):
-  """Compute Chern invariant"""
-  return mesh_chern(h,**kwargs) # workaround
-  # the wannier winding does not work
-  c = wannier_winding(h,full=True,**kwargs) 
-  open("CHERN.OUT","w").write(str(c))
-  return c
+    """Compute Chern invariant"""
+    return mesh_chern(h,**kwargs) # workaround
+    # the wannier winding does not work
+    c = wannier_winding(h,full=True,**kwargs) 
+    open("CHERN.OUT","w").write(str(c))
+    return c
 
 
 
 
-def wannier_winding(h,nk=40,nt=40,nocc=None,full=True):
-  m = z2_vanderbilt(h,nk=nk,nt=nt,nocc=nocc,full=full)
-  x = m[0]
-  # find the position of the maximum gap at every t
-  fermis = x*0. # maximum gap
-  for it in range(len(x)): # loop over times
-    imax,jmax = None,None
-    dmax = -1 # initialize
-    gapangle = None # initialize
-    maxgap = -1.0 # maximum gap
-    gapangle = ((m[1][it]+0.5)%1)*np.pi # initialize
-    for i in range(1,len(m)):
-      for j in range(i+1,len(m)):
-        for ipi in [0.,1.]:
-          ip = np.exp(1j*m[i][it]) # center of wave i
-          jp = np.exp(1j*m[j][it]) # center of wave j
-          angle = np.angle(ip+jp)+np.pi*ipi # get the angle
-          dp = np.exp(1j*angle) # now obtain this middle point gap
-          mindis = 4.0 # calculate minimum distance
-          for k in range(1,len(m)): # loop over centers
-            kp = np.exp(1j*m[k][it]) # center of wave k
-            dis = np.abs(dp-kp) # distance between the two points 
-            if dis<mindis: mindis = dis+0. # update minimum distance
-          if mindis>maxgap: # if found a bigger gap
-            maxgap = mindis+0. # maximum distance
-            gapangle = np.angle(dp) # update of found bigger gap
-    fermis[it] = gapangle
-  # now check the number of cuts of each wannier center
-  def angleg(a,b,c):
-    """Function to say if a jump has been made or not"""
-    d = np.sin(a-b) + np.sin(b-c) + np.sin(c-a)
-    return -d
-
-  if full: # for the Chern number
-    raise
-    # this part is wrong
-    #####################
-    cuts = 0 # start with 0
-#    print(fermis)
-    for i in range(1,len(m)): # loop over waves 
-      cwf = m[i] # center of the wave
-      for it in range(len(x)-1): # loop over times
-        s1 = np.sign(fermis[it]-cwf[it])
-        s2 = np.sign(fermis[it+1]-cwf[it])
-#        print(s1,s2)
-        cuts += (s1-s2)/2. 
-    return cuts
-
-  else: # for the Z2 invariant
-    parity = 1 # start with
-    for i in range(1,len(m)): # loop over waves 
-      cwf = m[i] # center of the wave
-      for it in range(len(x)-1): # loop over times
-        s = np.sign(angleg(fermis[it],fermis[it+1],cwf[it])) 
-        if s<0.:  parity *= -1 # add a minus sign
-    return parity
-
-
-
+def z2_wannier_winding(h,nk=100,nt=100,nocc=None,full=True):
+    """Compute the winding of the Wannier functions"""
+    m = z2_wannier_centers(h,nk=nk,nt=nt,nocc=nocc,full=full)
+    x = m[0]
+    from .topologytk.wannier import maximum_wannier_gap
+    fermis = maximum_wannier_gap(m)
+    # now check the number of cuts of each wannier center
+    def angleg(a,b,c):
+      """Function to say if a jump has been made or not"""
+      d = np.sin(a-b) + np.sin(b-c) + np.sin(c-a)
+      return -d
+  
+    if full: # for the Chern number
+      raise
+      # this part is wrong
+      #####################
+      cuts = 0 # start with 0
+  #    print(fermis)
+      for i in range(1,len(m)): # loop over waves 
+        cwf = m[i] # center of the wave
+        for it in range(len(x)-1): # loop over times
+          s1 = np.sign(fermis[it]-cwf[it])
+          s2 = np.sign(fermis[it+1]-cwf[it])
+  #        print(s1,s2)
+          cuts += (s1-s2)/2. 
+      return cuts
+  
+    else: # for the Z2 invariant
+      parity = 1 # start with
+      for i in range(1,len(m)): # loop over waves 
+        cwf = m[i] # center of the wave
+        for it in range(len(x)-1): # loop over times
+          s = np.sign(angleg(fermis[it],fermis[it+1],cwf[it])) 
+          if s<0.:  parity *= -1 # add a minus sign
+      return parity
 
 
 
+wannier_winding = z2_wannier_winding # for compatibility
 
 
 
 
 def operator_berry(hin,k=[0.,0.],operator=None,delta=0.00001,ewindow=None):
-  """Calculates the Berry curvature using an arbitrary operator"""
-  h = multicell.turn_multicell(hin) # turn to multicell form
-  dhdx = multicell.derivative(h,k,order=[1,0]) # derivative
-  dhdy = multicell.derivative(h,k,order=[0,1]) # derivative
-  hkgen = h.get_hk_gen() # get generator
-  hk = hkgen(k) # get hamiltonian
-  (es,ws) = algebra.eigh(hkgen(k)) # initial waves
-  ws = np.conjugate(np.transpose(ws)) # transpose the waves
-  n = len(es) # number of energies
-  from .berry_curvaturef90 import berry_curvature as bc90
-  if operator is None: operator = np.identity(dhdx.shape[0],dtype=np.complex)
-  b = bc90(dhdx,dhdy,ws,es,operator,delta) # berry curvature
-  return b*np.pi*np.pi*8 # normalize so the sum is 2pi Chern
+    """Calculates the Berry curvature using an arbitrary operator"""
+    h = multicell.turn_multicell(hin) # turn to multicell form
+    dhdx = multicell.derivative(h,k,order=[1,0]) # derivative
+    dhdy = multicell.derivative(h,k,order=[0,1]) # derivative
+    hkgen = h.get_hk_gen() # get generator
+    hk = hkgen(k) # get hamiltonian
+    (es,ws) = algebra.eigh(hkgen(k)) # initial waves
+    ws = np.conjugate(np.transpose(ws)) # transpose the waves
+    n = len(es) # number of energies
+    from .berry_curvaturef90 import berry_curvature as bc90
+    if operator is None: operator = np.identity(dhdx.shape[0],dtype=np.complex128)
+    b = bc90(dhdx,dhdy,ws,es,operator,delta) # berry curvature
+    return b*np.pi*np.pi*8 # normalize so the sum is 2pi Chern
 
 
 
 def operator_berry_bands(hin,k=[0.,0.],operator=None,delta=0.00001):
-  """Calculates the Berry curvature using an arbitrary operator"""
-  h = multicell.turn_multicell(hin) # turn to multicell form
-  dhdx = multicell.derivative(h,k,order=[1,0]) # derivative
-  dhdy = multicell.derivative(h,k,order=[0,1]) # derivative
-  hkgen = h.get_hk_gen() # get generator
-  hk = hkgen(k) # get hamiltonian
-  (es,ws) = algebra.eigh(hkgen(k)) # initial waves
-  ws = np.conjugate(np.transpose(ws)) # transpose the waves
-  from .berry_curvaturef90 import berry_curvature_bands as bcb90
-  if operator is None: operator = np.identity(dhdx.shape[0],dtype=np.complex)
-  bs = bcb90(dhdx,dhdy,ws,es,operator,delta) # berry curvatures
-  return (es,bs*np.pi*np.pi*8) # normalize so the sum is 2pi Chern
+    """Calculates the Berry curvature using an arbitrary operator"""
+    h = multicell.turn_multicell(hin) # turn to multicell form
+    dhdx = multicell.derivative(h,k,order=[1,0]) # derivative
+    dhdy = multicell.derivative(h,k,order=[0,1]) # derivative
+    hkgen = h.get_hk_gen() # get generator
+    hk = hkgen(k) # get hamiltonian
+    (es,ws) = algebra.eigh(hkgen(k)) # initial waves
+    ws = np.conjugate(np.transpose(ws)) # transpose the waves
+    from .berry_curvaturef90 import berry_curvature_bands as bcb90
+    if operator is None: operator = np.identity(dhdx.shape[0],dtype=np.complex128)
+    bs = bcb90(dhdx,dhdy,ws,es,operator,delta) # berry curvatures
+    return (es,bs*np.pi*np.pi*8) # normalize so the sum is 2pi Chern
 
 
 
@@ -479,11 +461,12 @@ from .topologytk.green import berry_operator
 
 
 
-def berry_green_map_kpoint(h,emin=None,k=[0.,0.,0.],
+def berry_green_rmap_kpoint(h,emin=None,k=[0.,0.,0.],
         ne=100,dk=0.0001,operator=None,integral_mode="complex",
                   delta=0.002,integral=True,eps=1e-1,
                   energy=0.0,emax=0.0):
-  """Return the Berry curvature map at a certain kpoint"""
+  """Return the Berry curvature map at a certain kpoint. This function
+  allows to compute both dOmega/dE, and Omega"""
   f = h.get_gk_gen(delta=delta,canonical_phase=True) # green function generator
   fgreen = berry_green_generator(f,k=k,dk=dk,operator=operator,full=True) 
   # No minimum energy provided
@@ -515,20 +498,22 @@ def berry_green_map_kpoint(h,emin=None,k=[0.,0.,0.],
 
 
 def spatial_berry_density(h,**kwargs):
-    return berry_green_map(h,integral=False,**kwargs)
+    """Berry density at the Fermi energy and spatially resolved"""
+    return Omega_rmap(h,integral=False,**kwargs)
 
 
 
-def berry_green_map(h,nrep=5,k=[0.,0.,0.],operator=None,nk=None,**kwargs):
+def Omega_rmap(h,nrep=5,k=[0.,0.,0.],operator=None,nk=None,**kwargs):
   """
-  Write the Berry curvature of a kpoint in a file
+  Write the spatial resolved Berry curvature of a kpoint in a file.
+  If nk is provided, it does a sum over reciprocal space
   """
   if operator is not None: # this is a dirty workaround
     if type(operator) is str:
       operator = h.get_operator(operator) 
     else: pass
   if nk is None: # kpoint given
-    out = berry_green_map_kpoint(h,k=k,operator=operator,**kwargs) 
+    out = berry_green_rmap_kpoint(h,k=k,operator=operator,**kwargs) 
   else: # kpoint not given
     if operator is not None: 
         from . import gauge
@@ -537,32 +522,29 @@ def berry_green_map(h,nrep=5,k=[0.,0.,0.],operator=None,nk=None,**kwargs):
     ks = klist.kmesh(h.dimensionality,nk=nk) # kpoints
     def f(ki):
         print("kpoint",ki)
-        return berry_green_map_kpoint(h,k=ki,operator=operator,**kwargs)
+        return berry_green_rmap_kpoint(h,k=ki,operator=operator,**kwargs)
     out = parallel.pcall(f,ks) # compute all
     out = np.mean(out,axis=0) # resum
   from . import geometry
   from .ldos import spatial_dos
   # write in a file
   geometry.write_profile(h.geometry,
-          spatial_dos(h,out),name="BERRY_MAP.OUT",nrep=nrep)
+          spatial_dos(h,out),name="BERRY_RMAP.OUT",nrep=nrep)
   return out
 
 
 
-def berry_density(h,k=[0.,0.,0.],operator=None,delta=0.02,dk=0.02):
-  """Compute the Berry density"""
-  f = h.get_gk_gen(delta=delta,canonical_phase=True) # green function generator
-  fgreen = berry_green_generator(f,k=k,dk=dk,operator=operator,full=False) 
-  return fgreen(0.0).real
+# alias for compatibility
+from .topologytk.green import dOmega_dE
 
 
-def berry_density_map(h,nk=40,reciprocal=True,nsuper=1,
+def dOmega_dE_kmap(h,nk=40,reciprocal=True,nsuper=1,
                delta=None,operator=None,dk=0.01):
-  """Compute a Berry density map"""
+  """Compute a Berry density map Omega/dE (k) at a fixed energy"""
   if delta is None: delta = 5./nk
   if reciprocal: R = h.geometry.get_k2K()
   else: R = np.matrix(np.identity(3))
-  fo = open("BERRY_DENSITY_MAP.OUT","w") # open file
+  fo = open("BERRY_DENSITY_KMAP.OUT","w") # open file
   nt = nk*nk # total number of points
   ik = 0
   ks = [] # list with kpoints
@@ -575,7 +557,7 @@ def berry_density_map(h,nk=40,reciprocal=True,nsuper=1,
       else: print("Doing",ki)
       r = np.matrix(ki).T # real space vectors
       k = np.array((R*r).T)[0] # change of basis
-      b = berry_density(h,k=k,operator=operator,dk=dk) # get the density
+      b = dOmega_dE(h,k=k,operator=operator,dk=dk) # get the density
       return b
   bs = parallel.pcall(fp,ks) # compute all the Berry curvatures
   for (b,k) in zip(bs,ks): # write everything
@@ -586,6 +568,7 @@ def berry_density_map(h,nk=40,reciprocal=True,nsuper=1,
 
 
 def chern_density(h,nk=10,operator=None,delta=0.02,dk=0.02,
+        write=False,
         es=np.linspace(-1.0,1.0,40)):
   """Compute the Chern density as a function of the energy"""
   ks = klist.kmesh(h.dimensionality,nk=nk)
@@ -598,14 +581,16 @@ def chern_density(h,nk=10,operator=None,delta=0.02,dk=0.02,
     else: print(k)
 #    k = np.random.random(3)
     fgreen = berry_green_generator(f,k=k,dk=dk,operator=operator,full=False) 
-    return np.array([fgreen(e).real for e in es])
+    return np.array([fgreen(e).real for e in es]) # dOmega/dE at all energies
   out = parallel.pcall(fp,ks) # compute everything
   for o in out: cs += o # add contributions
   cs = cs/(len(ks)*np.pi*2) # normalize
   from scipy.integrate import cumtrapz
   csi = cumtrapz(cs,x=es,initial=0) # integrate
-  np.savetxt("CHERN_DENSITY.OUT",np.matrix([es,cs]).T)
-  np.savetxt("CHERN_DENSITY_INTEGRATED.OUT",np.matrix([es,csi]).T)
+  if write:
+      np.savetxt("CHERN_DENSITY.OUT",np.matrix([es,cs]).T)
+      np.savetxt("CHERN_DENSITY_INTEGRATED.OUT",np.matrix([es,csi]).T)
+  return (es,cs,csi)
 
 
 
@@ -634,3 +619,6 @@ from .topologytk import realspace
 
 real_space_chern = realspace.real_space_chern
 
+
+from .topologytk.topologicalsector import get_berry_curvature_operator_sector
+from .topologytk.topologicalsector import get_chern_operator_sector
