@@ -3,6 +3,7 @@ from .. import algebra
 from ..green import green_renormalization
 from .. import green
 from copy import deepcopy
+from ..htk.mode import make_compatible
 
 delta_smatrix = 1e-12
 dagger = algebra.dagger
@@ -13,8 +14,8 @@ gfmode = "adaptive"
 
 class LocalProbe():
     def __init__(self,h,lead=None,delta=1e-5,i=0,T=1.0,**kwargs):
+        h = h.get_dense() # dense Hamiltonian
         self.H = h.copy() # store Hamiltonian
-        self.H.turn_dense() # dense Hamiltonian
         self.has_eh = self.H.has_eh # electron-hole
         self.delta = delta
         self.mode = "bulk"
@@ -26,8 +27,7 @@ class LocalProbe():
         if lead is None:
             from ..geometry import chain 
             lead = chain().get_hamiltonian(has_spin=False) # create a chain
-            if self.H.has_spin: lead.turn_spinful()
-            if self.H.has_eh: lead.turn_nambu()
+        lead = make_compatible(lead,self.H) # make them compatible
         lead = lead.get_no_multicell() # no multicell
         self.lead = lead.copy() # store
         self.get_eh_sector = self.lead.get_eh_sector # if it has electron-hole
@@ -59,6 +59,8 @@ class LocalProbe():
         from .kappa import get_kappa_ratio
         if T is None: T = self.T 
         return get_kappa_ratio(self,T=T,**kwargs)
+    def get_dos(self,**kwargs):
+        return get_dos_bulk(self,**kwargs)
 
 
 
@@ -97,14 +99,14 @@ def local_selfenergy(h,g,energy=0.0,i=0,delta=1e-5,**kwargs):
     M = get_intra(h) # get intracell matrix
     gi = local_hamiltonian(h,g,i=i) # local Green's function
     oi = local_hamiltonian(h,M,i=i) # local Hamiltonian
-    iden = np.identity(gi.shape[0],dtype=np.complex)
+    iden = np.identity(gi.shape[0],dtype=np.complex128)
     out = algebra.inv(gi) - (energy+1j*delta)*iden + oi # local selfenergy
     return -out
 
 
 
 def get_central_gmatrix(P,selfl=None,selfr=None,energy=0.0):
-    """Return the central Green's function"""
+    """Return the (inverse) central Green's function"""
     delta = P.delta # imaginary part
     if selfl is None: selfl = P.get_selfenergy(lead=0,energy=energy)
     if selfr is None: selfr = P.get_selfenergy(lead=1,energy=energy)
@@ -126,6 +128,23 @@ def get_central_gmatrix(P,selfl=None,selfr=None,energy=0.0):
     return hlist
 
 
+
+def get_dos_bulk(self,operator="electron",**kwargs):
+    """Return the DOS of the bulk of a local probe object"""
+    g = self.get_central_gmatrix(**kwargs) # return Green's function
+    from ..green import gauss_inverse
+    g11 = gauss_inverse(g,1,1)
+    O = self.H.get_operator(operator)
+    if O is not None:
+        O = O*self.H.get_operator("site",index=0) # on a single site
+        O = O.get_matrix() # get the matrix
+        g11 = O@g11
+    return -np.trace(g11.imag)/np.pi
+
+
+
+
+
 def get_reflection_normal_lead(P,s):
     return s[0][0]
 
