@@ -10,6 +10,7 @@ def get_smatrix(ht,energy=0.0,as_matrix=False,check=True):
     """Calculate the S-matrix of an heterostructure"""
     # now do the Fisher Lee trick
     delta = ht.delta
+    if delta>delta_smatrix: delta = delta_smatrix # small delta is critical!
     smatrix = [[None,None],[None,None]] # smatrix in list form
     # get the selfenergies, using the same coupling as the lead
     selfl = ht.get_selfenergy(energy,delta=delta,lead=0,pristine=True)
@@ -73,8 +74,7 @@ def effective_tridiagonal_hamiltonian(intra,selfl,selfr,
     if not type(intra) is list: raise # assume is list
     n = len(intra) # number of blocks
     iout = [[None for i in range(n)] for j in range(n)] # empty list
-    iden = np.matrix(np.identity(intra[0][0].shape[0],dtype=np.complex))
-    if delta>delta_smatrix: delta = delta_smatrix # small delta is critical!
+    iden = np.matrix(np.identity(intra[0][0].shape[0],dtype=np.complex128))
     ez = iden*(energy +1j*delta) # complex energy
     for i in range(n):
       iout[i][i] = ez - intra[i][i] # simply E -H
@@ -146,10 +146,10 @@ def build_effective_hlist(ht,energy=0.0,delta=0.0001,selfl=None,selfr=None):
     if len(ht.central_intra)==0: # no central part provided
         ht.central_intra = (ht.left_intra + ht.right_intra)/2.
         print("Generating a dummy central cell, you may want to use a different geometry")
-    ce = energy +1j*delta
-    idenc = np.matrix(np.identity(len(ht.central_intra),dtype=complex))*ce
-    idenl = np.matrix(np.identity(len(ht.left_intra),dtype=complex))*ce
-    idenr = np.matrix(np.identity(len(ht.right_intra),dtype=complex))*ce
+    ce = energy +1j*delta # complex energy
+    idenc = np.matrix(np.identity(ht.central_intra.shape[0],dtype=complex))*ce
+    idenl = np.matrix(np.identity(ht.left_intra.shape[0],dtype=complex))*ce
+    idenr = np.matrix(np.identity(ht.right_intra.shape[0],dtype=complex))*ce
     hlist = [[None for i in range(3)] for j in range(3)] # list of matrices
     # set up the different elements
     # first the intra terms
